/*
 *  TombstoneCamera.h
 *  FreeFalling
 *
 *  Created by EdzUp on 07/05/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

/*
	Camera system for Tombstone, this will handle all camera position and rotation and state changes
 */

#if defined( __APPLE__ )					//Apple platform uses OpenGL ES
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
#import <UIKit/UIKit.h>
#import <OpenGLES/ES1/gl.h>
#import "TombstoneiPhoneFunctions.h"
#else
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <GLUT/GLUT.h>
#import "TombstoneOSXFunctions.h"
#endif
#endif

#if defined( __LINUX__ )
#include <GL/glut.h>    // Header File For The GLUT Library 
#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#include <unistd.h>     // Header File For sleeping.
#import "TombstoneLinuxFunctions.h"		//all linux declarations for the entire engine
#endif

#if defined( __WINDOWS__ )
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library#import <windows.h>						//on windows platform import the Windows headers
#import "TombstoneWindowsFunctions.h"	//windows functions
#endif

#include <iostream>
#include <string>
#include <list>
#import "TombstoneGlu.h"				//required for ALL MATH stuff
using namespace std;

class cameraClass;						//declaration so the list doesnt chuck a error

static long cameraCount;				//static as only ONE variable is required not hundreds ;)

class cameraClass{
private:
	GLuint	Id;							//Id of camera
	GLfloat X, Y, Z;					//location in 3d space of camera
	GLfloat Pitch, Yaw, Roll;			//rotation of camera in 3d space
	
	GLfloat AspectRatio;
	GLfloat FieldOfView;
	GLint	viewX, viewY;				//view X and Y coordinates
	GLsizei viewWidth, viewHeight;		//viewport width and height
	GLfloat zoomNear, zoomFar;			//GL Camera zoom system
	GLfloat rangeNear, rangeFar;		//camera ranges
	
	short	active;						//TRUE if this camera is active

	gluClass TombstoneGlu;				//Glu implementation
	
public:	
	//======================================================================================================
	//======================================================================================================
	void SetupCamera( void ) {
		//creates a camera for usage
		Id = cameraCount;
		X = 0.0;
		Y = 0.0;
		Z = 0.0;
		Pitch = 0.0;
		Yaw = 0.0;
		Roll = 0.0;
		viewX = 0;
		viewY = 0;
		viewWidth = 320;
		viewHeight = 480;
		zoomNear = 0.1;
		zoomFar = 500.0;
		rangeNear = 0.1;
		rangeFar = 500.0;
		FieldOfView = 50.0;
		AspectRatio = 0.75;
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		TombstoneGlu.gluPerspective( FieldOfView, AspectRatio, rangeNear, rangeFar );
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		TombstoneGlu.gluLookAt(0.0, 0.0, 0.0,
							   0.0, 0.0, 0.0,
							   0.0, 1.0, 0.0);
		
		cameraCount++;								//increment camera count
	}

	//======================================================================================================
	void debugCameraInformation( void ) {
		printf( "<< DEBUG CAMERA INFORMATION >>\n" );
		printf( "Camera ID: %i\n", Id );
		printf( "Coords: X:%f Y:%f Z:%f\n", X, Y, Z );
		printf( "Rotation: Pitch:%f Yaw:%f Roll:%f\n", Pitch, Yaw, Roll );
		printf( "View: X:%i Y:%i Width:%i Height:%i\n", viewX, viewY, viewWidth, viewHeight );
		printf( "Zoom: Near:%f Far:%f\n", zoomNear, zoomFar );
		printf( "Range: Near:%f Far:%f\n", rangeNear, rangeFar );
		printf( "Field of view:%f\n", FieldOfView );
		printf( "Aspect Ratio:%f\n", AspectRatio );
		printf( "<< END OF DEBUG CAMERA INFORMATION >>\n\n" );
	}
	
	//======================================================================================================
	void Render( void ) {
		//this will render the whole camera setup
		/*
		 Ya, it's possible. You render the view from each camera sequentially. To switch to a new camera, change your viewport, projection and modelview matrices to whatever's appropriate. The viewport is not guaranteed to clip pixels, so if any of your rendering might fall outside the viewport, be sure to use glScissor().
		 
		 Example:
		 
		 // prelims
		 glEnable(GL_SCISSOR_TEST);
		 
		 // camera 1
		 glViewport(...);
		 glScissor(...); // match viewport extents (X, Y, Width, Height )
		 glMatrixMode(GL_PROJECTION);
		 gluOrtho(...);
		 glMatrixMode(GL_MODELVIEW);
		 glTranslatef(...);
		 glRotatef(...);
		 ...(rendering for camera 1)
		  then camera 2 etc.
		 
		 Hope this helps.
		 */
		glViewport( viewX, viewY, viewWidth, viewHeight );
		glScissor( viewX, viewY, viewWidth, viewHeight );
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		TombstoneGlu.gluPerspective(FieldOfView, AspectRatio, rangeNear, rangeFar );
			//gluOrtho...
		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();
		
		debugCameraInformation( );
		TombstoneGlu.gluLookAt( X, Y, Z, 0,0,0, 0, 1, 0 );
		/*
		 glMatrixMode(GL_PROJECTION);
		 glLoadIdentity();
		 TombstoneGlu.gluPerspective(50.0, .75, 0.01, 1000.0);
		 glMatrixMode(GL_MODELVIEW);
		 glLoadIdentity();
		 TombstoneGlu.gluLookAt(0.0, 0.0, 450.0,
		 0.0, 0.0, 0.0,
		 0.0, 1.0, 0.0);		 
		 */
	}

	//======================================================================================================
	void DisableCamera( void ) {
		//remove the camera from render situation
		active = FALSE;
	}
	
	//======================================================================================================
	void EnableCamera( void ) {
		active = TRUE;
	}
	
	//======================================================================================================
	void SetCameraFOV( GLfloat FOV, GLfloat Aspect = 0.75 ) {
		FieldOfView = FOV;
		AspectRatio = Aspect;
	}

	//======================================================================================================
	void SetCameraViewport( GLint vX, GLint vY, GLsizei vWidth, GLsizei vHeight ) {
		viewX = vX;
		viewY = vY;
		viewWidth = vWidth;
		viewHeight = vHeight;
	}

	//======================================================================================================
	void PositionCamera( GLfloat pX, GLfloat pY, GLfloat pZ ) {
		//this will position a camera at a specific location
		X = pX;
		Y = pY;
		Z = pZ;
	}

	//======================================================================================================
	void TurnCamera( GLfloat pitch, GLfloat yaw, GLfloat roll ) {
		Pitch += pitch;
		if ( Pitch >360.0 ) Pitch -=360;
		if ( Pitch <0.0 ) Pitch +=360;
		Yaw += yaw;
		if ( Yaw >360.0 ) Yaw -=360;
		if ( Yaw <0.0 ) Yaw +=360;
		Roll += roll;
		if ( Roll >360.0 ) Roll -=360;
		if ( Roll <0.0 ) Roll +=360;
	}
	
	//======================================================================================================
	void RotateCamera( GLfloat rPitch, GLfloat rYaw, GLfloat rRoll ) {
		//rotate the camera
		Pitch = rPitch;
		Yaw = rYaw;
		Roll = rRoll;
	}
	
	//======================================================================================================
	void SetCameraZoom( GLfloat pNear, GLfloat pFar ) {
		//set the camera zoom level
		zoomNear = pNear;
		zoomFar = pFar;
	}
	
	//======================================================================================================
	void SetCameraRange( GLfloat rNear, GLfloat rFar ) {
		//sets the camera ranges
		rangeNear = rNear;
		rangeFar = rFar;
	}
};

typedef cameraClass TCamera;
