/*
 *  TColour.h
 *  Treasure Caves
 *
 *  Created by EdzUp on 26/08/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

#import "TombstoneVariables.h"

class TColourClass {
public:
	long Red;
	long Green;
	long Blue;
	long Alpha;
};

typedef TColourClass TColour;

class TDataBankClass {
public:
	unsigned char*	data;
	long	size;
	GLuint	width;
	GLuint	height;
	
	//======================================================================================================
	void extractImageRGBAData( const std::string& filename ) {
		// Turn necessary features on
		//		glEnable(GL_TEXTURE_2D);

#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
		//convert string to NSSTring
		NSString	*NSfilename = iPhoneFilename( filename.c_str() );
		NSData		*texData = [[NSData alloc] initWithContentsOfFile:NSfilename];
		UIImage		*image = [[UIImage alloc] initWithData:texData];
		if (image == nil) {
			//			cout << "Texture loader error: " << convertNSStringToChar( NSfilename ) << " could not be loaded." << endl;
			exit( 1 );
		}
		
		GLuint width = CGImageGetWidth(image.CGImage);
		GLuint height = CGImageGetHeight(image.CGImage);
		CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
		data = ( unsigned char *)malloc( height * width * 4 );
		CGContextRef context = CGBitmapContextCreate( data, width, height, 8, 4 * width, colorSpace, kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big );
		CGColorSpaceRelease( colorSpace );
		CGContextClearRect( context, CGRectMake( 0, 0, width, height ) );
		CGContextTranslateCTM( context, 0, height - height );
		CGContextDrawImage( context, CGRectMake( 0, 0, width, height ), image.CGImage );
		
		size = ( width *height ) *4;		//store size
		width = width;					//store width
		height = height;					//store height
		
		CGContextRelease(context);
		
		[image release];
		[texData release];
#else
#endif
	}
	
	//======================================================================================================
	void Plot( long X, long Y, TColour colour, short ignoreAlpha = TRUE ) {
		//this will plot to a certain colour
		if ( GLRed != 0 ) {
			data[ ( Y *( width *4 ) ) +( X *4 ) ] = colour.Red;
		} else {
			data[ ( Y *( width *4 ) ) +( X *4 ) ] = 0;
		}
		if ( GLGreen != 0 ) {
			data[ ( ( Y *( width *4 ) ) +( X *4 ) ) +1 ] = colour.Green;
		} else {
			data[ ( ( Y *( width *4 ) ) +( X *4 ) ) +1 ] = 0;
		}
		if ( GLBlue != 0 ) {
			data[ ( ( Y *( width *4 ) ) +( X *4 ) ) +2 ] = colour.Blue;
		} else {
			data[ ( ( Y *( width *4 ) ) +( X *4 ) ) +2 ] = 0;
		}
		if ( ignoreAlpha == FALSE ) {
			//if we dont want to ignore alpha then we can use it
			if ( GLAlpha != 0 ) {
				data[ ( ( Y *( width *4 ) ) +( X *4 ) ) +3 ] = colour.Alpha;
			} else {
				data[ ( ( Y *( width *4 ) ) +( X *4 ) ) +3 ] = 0;
			}
		}
	}
	
	
	//======================================================================================================
	void replaceColour( TColour from, TColour to, short ignoreAlpha = TRUE ) {
		//This will run through the entire allocated array and replace the colour with the to colour
		for ( long rP = 0; rP <size; rP +=4 ) {
			//get the colour from the array and replace it
			if ( ignoreAlpha == FALSE ) {
				if ( data[ rP ] == from.Red && data[ rP +1 ] == from.Green && data[ rP +2 ] == from.Blue && data[ rP +3 ] == from.Alpha ) {
					//replace everything
					
					data[ rP ] = to.Red;
					data[ rP +1 ] = to.Green;
					data[ rP +2 ] = to.Blue;
					data[ rP +3 ] = to.Alpha;
				}
			} else {
				if ( data[ rP ] == from.Red && data[ rP +1 ] == from.Green && data[ rP +2 ] == from.Blue ) {
					//replace only colour
					data[ rP ] = to.Red;
					data[ rP +1 ] = to.Green;
					data[ rP +2 ] = to.Blue;
				}
			}
		}
	}
	
	//======================================================================================================
	void releaseDataBank( void ) {
		//this will erase the databank
		free( data );
		width = 0;
		height = 0;
	}
};
typedef TDataBankClass TDataBank;

//======================================================================================================
TColour RGBA( long red, long green, long blue, long alpha ) {
	TColour temp;
	
	temp.Red = red;
	temp.Green = green;
	temp.Blue = blue;
	temp.Alpha = alpha;
	
	return( temp );
}
