/*
 *  TEntity.h
 *  Tombstone Demo
 *
 *  Created by EdzUp on 25/06/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */
#include <fstream>
#include <iostream>
#include <string>
#include <list>
#include <vector>
#import "TRegistry.h"
#import "TTexture.h"
#import "TombstoneGlu.h"				//required for ALL MATH stuff
#import "TSurface.h"
#import "TombstoneDebug.h"
#import "TombstoneConversion.h"

#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
#include "TombstoneiPhoneFunctions.h"
#endif

using namespace std;

class TagData {
public:
	string name;
	
	GLfloat Red;
	GLfloat Green;
	GLfloat Blue;
	long colourEnvelope;
};
typedef TagData Tag;

class FaceData {
public:
	long VertexCount;
	long Vertex[ 4 ];
	
	long SurfaceID;
};
typedef FaceData Face4d;

class VertexData {
public:
	GLfloat X;
	GLfloat Y;
	GLfloat Z;
	
	GLfloat U;
	GLfloat V;
};
typedef VertexData Vertex3d;

class TEntityClass;
typedef TEntityClass TEntity;
long gNumEntities;
TDebug	EntityDebug;

list <TEntity*> TEntityRegistry;

class TEntityClass {
public:
	long	ID;								//this is the Tombstone ID relating to the entry
	TTexture	name;							//this is the OpenGL reference for the item
	GLfloat X, Y, Z;						//position
	GLfloat Xr, Yr, Zr;						//rotation
	GLfloat Red, Green, Blue, Alpha;		//colours of the mesh
	long	surfaceCount;
	short	hidden;							//True if this entity is hidden
	vector<TSurface*> Surface;				//surface list for Entity

	//states
	short	fullBright;
	short	vertexColour;
	short	flatShaded;
	short	disableFog;
	short	backfaceCulling;				//false is backfaces arent to be drawn
	short	alphaBlending;	
	
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
	//Iphone specific code here
	NSData* readFileFromBundle( const char* filename, char* extension ) {
		NSString *newname = [NSString stringWithUTF8String:filename];
		NSString *newnameext = [NSString stringWithUTF8String:extension ];
		NSString *filePath = [[NSBundle mainBundle] pathForResource:newname ofType:newnameext ];  
		NSData *myData = [NSData dataWithContentsOfFile:filePath];		
		return( myData );
	}
#endif

	//======================================================================================================
	void deleteRegistryEntry( void ) {
		//remove it from the registry list, this is called by FreeEntity
		list< TEntity* >::iterator i;
		
		for ( i =TEntityRegistry.begin(); i !=TEntityRegistry.end(); i++ ){
			TEntity* GItemp = *i;
			if ( GItemp->ID == ID ) {
				//need to copy the data segment
				TEntityRegistry.erase( i );
				return;
			}
		}		
	}

	//======================================================================================================
	void updateRegistry( void ) {
		//run through registry list and update the entry relating to this TEntity
		TEntity* temp = new TEntity;
		
		list< TEntity* >::iterator i;
		
		for ( i =TEntityRegistry.begin(); i !=TEntityRegistry.end(); i++ ){
			TEntity* GItemp = *i;
			if ( GItemp->ID == ID ) {
				//need to copy the data segment
				temp = GItemp;
				TEntityRegistry.erase( i );
			}
		}
		
		temp->ID = ID;
		temp->name = name;
		temp->X = X;
		temp->Y = Y;
		temp->Z = Z;
		temp->Xr = Xr;
		temp->Yr = Yr;
		temp->Zr = Zr;
		temp->Red = Red;
		temp->Green = Green;
		temp->Blue = Blue;
		temp->Alpha = Alpha;
		temp->surfaceCount = surfaceCount;
		temp->hidden = hidden;
		temp->Surface = Surface;
		temp->fullBright = fullBright;
		temp->vertexColour = vertexColour;
		temp->flatShaded = flatShaded;
		temp->disableFog = disableFog;
		temp->backfaceCulling = backfaceCulling;
		temp->alphaBlending = alphaBlending;
		TEntityRegistry.push_back( temp );
			
//		cout << "Updated " << temp->ID << endl;
		
			return;			//exit function
//		}
		
		printf( "No entity list information with ID %li\n", ID );
		exit( 1 );
	}

	//======================================================================================================
	void EntityFX( long fx ) {
		/*
		 EntityFX entity,fx
		 
		 Parameters:
		 
		 entity - entity handle
		 
		 fx -
		 0: nothing (default)
		 1: full-bright
		 2: use vertex colors instead of brush color
		 4: flatshaded
		 8: disable fog
		 16: disable backface culling
		 32: force alpha-blending
		 Description:
		 
		 Sets miscellaneous effects for an entity.
		 
		 Flags can be added to combine two or more effects. For example, specifying a flag of 3 (1+2) will result in a full-bright and vertex-coloured brush.
		 
		 Flag 32, to force alpha-blending, must be used in order to enable vertex alpha (see VertexColor).
		 
		 See also: VertexColor.
		 
		 //states
		 short	fullBright;
		 short	vertexColour;
		 short	flatShaded;
		 short	disableFog;
		 short	backfaceCulling;
		 short	alphaBlending;			 
		 */
		
		long Efx = fx;
		
		//disable everything
		alphaBlending = FALSE;
		backfaceCulling = FALSE;
		disableFog = FALSE;
		flatShaded = FALSE;
		vertexColour = FALSE;
		fullBright = FALSE;
		
		//enable stuff
		if ( Efx -32 >-1 ) {
			alphaBlending = TRUE;
			Efx -=32;
		}
		if ( Efx -16 >-1 ) {
			backfaceCulling = TRUE;
//			cout << "backface" << endl;
			Efx -=16;
		}
		if ( Efx -8 >-1 ) {
			disableFog = TRUE;
			Efx -=8;
		}
		if ( Efx -4 >-1 ) {
			flatShaded = TRUE;
			Efx -= 4;
		}
		if ( Efx -2 >-1 ) {
			vertexColour = TRUE;
			Efx -= 2;
		}
		if ( Efx -1 >-1 ) {
			fullBright = TRUE;
			Efx -= 1;
		}
	}

	//======================================================================================================
	void CreateEntity( void ) {
		ID = gNumEntities;
		
		hidden = FALSE;
		
		//default X,Y,Z to center of 3d space
		X = 0;
		Y = 0;
		Z = 0;
		
		//rotate to straight ahead
		Xr = 0;
		Yr = 0;
		Zr = 0;
		
		//full bright colour
		Red =	1.0;
		Green =	1.0;
		Blue =	1.0;
		Alpha =	1.0;
		
		fullBright = FALSE;
		vertexColour = FALSE;
		flatShaded = FALSE;
		disableFog = FALSE;
		backfaceCulling = FALSE;
		alphaBlending = FALSE;
		
		gNumEntities++;
	}
	
	//======================================================================================================
	long CreateSurface( void ) {
		TSurface* tempSurface = new TSurface;
		tempSurface->Id = surfaceCount;
		tempSurface->vertexPoints = 0;
		tempSurface->texturePoints = 0;
		tempSurface->fxFullBright = FALSE;
		tempSurface->fxVertexColour = FALSE;
		tempSurface->fxFlatShaded = FALSE;
		tempSurface->fxFogDisabled = FALSE;
		tempSurface->fxBackfaceDisabled = FALSE;
		tempSurface->fxAlphaBlending = FALSE;
		Surface.push_back( tempSurface );
		
		surfaceCount++;
		
		return( tempSurface->Id );
	}
	
	//======================================================================================================
	void freeEntity( void ) {
		//free the entity from memory
		Surface.clear();
		deleteRegistryEntry();
	}

	//======================================================================================================
	void hideEntity( void ) {
		hidden = TRUE;
	}
	
	//======================================================================================================
	void showEntity( void ) {
		hidden = FALSE;
	}
	
	//======================================================================================================
	void EntityScale( GLfloat esX, GLfloat esY, GLfloat esZ ) {
		//rescale an entity
		for ( long surfID = 0; surfID < surfaceCount; surfID++ ) {
			for ( long vertID = 0; vertID < Surface[ surfID ]->vertexPoints; vertID++ ) {
				Surface[ surfID ]->verts[ ( vertID *3 ) ] *= esX;
				Surface[ surfID ]->verts[ ( vertID *3 ) +1 ] *= esY;
				Surface[ surfID ]->verts[ ( vertID *3 ) +2 ] *= esZ;
			}
		}
	}
	
	//======================================================================================================
	void EntityAlpha( GLfloat alpha ) {
		Alpha = alpha;//( 1.0 /255 ) *alpha;
			
		//run through surfaces changing the colour
		for ( long surfID = 0; surfID < surfaceCount; surfID++ ) {
			for ( long vertID = 0; vertID < Surface[ surfID ]->vertexPoints; vertID++ ) {
				Surface[ surfID ]->vertexColours[ ( vertID *4 ) +3 ] = alpha;
			}
		}
	}
	
	//======================================================================================================
	void EntityColour( GLfloat red, GLfloat green, GLfloat blue ) {
		Red = ( 1.0 /255 ) *red;
		Green = ( 1.0 /255 ) *green;
		Blue = ( 1.0 /255 ) *blue;
			
		//run through surfaces changing the colour
		for ( long surfID = 0; surfID < surfaceCount; surfID++ ) {
			for ( long vertID = 0; vertID < Surface[ surfID ]->vertexPoints; vertID++ ) {
				Surface[ surfID ]->vertexColours[ vertID *4 ] = ( 1.0 /255.0 ) *red;
				Surface[ surfID ]->vertexColours[ ( vertID *4 ) +1 ] = ( 1.0 /255.0 ) *green;
				Surface[ surfID ]->vertexColours[ ( vertID *4 ) +2 ] = ( 1.0 /255.0 ) *blue;
			}
		}
	}
/*
	//======================================================================================================
	void updateTexture( GLuint newTexture ) {
		//required for adjustment system
		name = newTexture;
	}	
*/	
	//======================================================================================================
	void VertexCoords( long surfaceID, long vertexID, GLfloat ToX, GLfloat ToY, GLfloat ToZ, long mode = VERTEX_SPECIFIED ) {
		/*
		 1) get the X, Y, Z of the vertices
		 2) change it to the new target
		 3) if VERTEX_ALL specified run through ALL surfaces and vertices changing them as required.
		 */
		if ( mode == VERTEX_SPECIFIED ) {
			//adjust the specified verts
			Surface[ surfaceID ]->verts[ vertexID *3 ] = ToX;
			Surface[ surfaceID ]->verts[ ( vertexID *3 ) +1 ] = ToY;
			Surface[ surfaceID ]->verts[ ( vertexID *3 ) +2 ] = ToZ;
		} else {
			GLfloat FromX;
			GLfloat FromY;
			GLfloat FromZ;
			
			FromX = Surface[ surfaceID ]->verts[ vertexID *3 ];
			FromY = Surface[ surfaceID ]->verts[ ( vertexID *3 ) +1 ];
			FromZ = Surface[ surfaceID ]->verts[ ( vertexID *3 ) +2 ];
			for ( long surfID = 0; surfID < surfaceCount; surfID++ ) {
				for ( long vertID = 0; vertID < Surface[ surfID ]->vertexPoints; vertID++ ) {
					if ( Surface[ surfID ]->verts[ vertID *3 ] == FromX &&
						Surface[ surfID ]->verts[ ( vertID *3 ) +1 ] == FromY &&
						Surface[ surfID ]->verts[ ( vertID *3 ) +2 ] == FromZ ) {
						//change the vertex to the new coordinates
						Surface[ surfID ]->verts[ vertID *3 ] = ToX;
						Surface[ surfID ]->verts[ ( vertID *3 ) +1 ] = ToY;
						Surface[ surfID ]->verts[ ( vertID *3 ) +2 ] = ToZ;
					}
				}
			}
		}
	}
	
	//======================================================================================================
	void EntityTexture( TTexture texture, long surfaceID = -1, long textureIndex = -1 ) {
		//this will texture the Entity
		if ( surfaceID == -1 ) {
			name = texture;			//change it to the new texture
		} else {
			//texture that surface with the texture
			if ( surfaceID >-1 ) {
				if ( textureIndex >-1 && textureIndex<8 ) {
					//texture a specific surface and texture index.
					Surface[ surfaceID ]->textureID[ textureIndex ] = texture;
				}
			}
		}
	}

	//======================================================================================================
	void CreateQuad( void ) {
		//create a basic Quad
		long tempSurface;
		
		//begin entity creation
		CreateEntity();

		//create a openGL quad
		tempSurface = CreateSurface();
		Surface[ tempSurface ]->AddVertex(  1.0, -1.0,  0.0, 1, 1 );
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0,  0.0, 0, 1 );
		Surface[ tempSurface ]->AddVertex(  1.0,  1.0,  0.0, 1, 0 );
		Surface[ tempSurface ]->AddVertex(  1.0,  1.0,  0.0, 1, 0 );
		Surface[ tempSurface ]->AddVertex( -1.0,  1.0,  0.0, 0, 0 );
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0,  0.0, 0, 1 );
		
		//create the triangles
		Surface[ tempSurface ]->AddTriangle( 2, 1, 0 );
		Surface[ tempSurface ]->AddTriangle( 3, 4, 5 );
		
		Surface[ tempSurface ]->texturePoints = 3;
		
		updateRegistry();
	}

	//======================================================================================================
	void CreateSheet( int horizontalSlices =1, int verticalSlices =1 ) {
		//create a sheet for of quads
		long tempSurface;
		GLfloat Uinc = 2.0 /horizontalSlices;
		GLfloat Vinc = 2.0 /verticalSlices;
		GLfloat tempU = 0.0;
		GLfloat tempV = 0.0;
		
		long Vert1;
		long Vert2;
		long Vert3;
		long Vert4;
		long Vert5;
		long Vert6;
		//begin entity creation
		CreateEntity();
		
		cout << "CREATE SHEET:" << endl;
		cout << "U:" << Uinc << "  V:" << Vinc << endl;
		
		//create the sheet
		tempSurface = CreateSurface();
		for ( GLfloat zPos =-1.0; zPos <1.0; zPos += Vinc ) {
			for ( GLfloat xPos =-1.0; xPos <1.0; xPos += Uinc ) {
				//create Quad sheet here
				//vertices need to be -1.0 (size is -1 to 1 so -1.0 for a item thats 2.0 would be perfect :D
				Vert1 = Surface[ tempSurface ]->AddVertex( xPos,		0.0, zPos,		1, 1 );
				Vert2 = Surface[ tempSurface ]->AddVertex( xPos+Uinc,	0.0, zPos,		0, 1 );
				Vert3 = Surface[ tempSurface ]->AddVertex( xPos,		0.0, zPos+Vinc, 1, 0 );
				Vert4 = Surface[ tempSurface ]->AddVertex( xPos+Uinc,	0.0, zPos,		1, 0 );
				Vert5 = Surface[ tempSurface ]->AddVertex( xPos+Uinc,	0.0, zPos+Vinc,	0, 0 );
				Vert6 = Surface[ tempSurface ]->AddVertex( xPos,		0.0, zPos+Vinc,	0, 1 );
				
				//create the triangles
				Surface[ tempSurface ]->AddTriangle( Vert3, Vert2, Vert1 );
	//			Surface[ tempSurface ]->AddTriangle( Vert4, Vert5, Vert6 );
				
				Surface[ tempSurface ]->texturePoints = 3;
				
				cout << "VertU:" << xPos << "   VertZ:" << zPos << endl;
				
//				UpdateNormals( Mesh )
				
//				EntityColor Mesh, 150, 200, 255
				
				//setup the UV's so the entire thing is textured to one texture
				tempU += Uinc;
			}
			tempU = 0.0;
			tempV += Vinc;
		}
		
		cout << "CreateSheet::Vertex Count:" << Surface[ tempSurface ]->vertexPoints;
		
		updateRegistry();
	}

	//======================================================================================================
	void CreateCube( void ) {
		//this function will create a cube
		long tempSurface;
		
		CreateEntity();
		
		tempSurface = CreateSurface();
		//Front Face
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, 1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, 1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, 1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 0, 1, 2 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, 1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, 1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, 1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 3, 4, 5 );
		
		//Back Face
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, -1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, -1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, -1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 6, 7, 8 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, -1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, -1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, -1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 9, 10, 11 );
		
		//Top Face
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, -1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, 1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, -1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 12, 13, 14 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, 1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, 1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, -1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 15, 16, 17 );
		
		//Bottom Face
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, -1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, -1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, 1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 18, 19, 20 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, -1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, 1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, 1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddTriangle( 21, 22, 23 );
		
		//Right face
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, -1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, -1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, 1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddTriangle( 24, 25, 26 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, -1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, 1.0, 1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( 1.0, -1.0, 1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddTriangle( 27, 28, 29 );
		
		//Left face
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, -1.0, 0.0, 0.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, 1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, -1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddTriangle( 30, 31, 32 );
		Surface[ tempSurface ]->AddVertex( -1.0, -1.0, 1.0, 0.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, 1.0, 1.0, 1.0 );
		Surface[ tempSurface ]->AddVertex( -1.0, 1.0, -1.0, 1.0, 0.0 );
		Surface[ tempSurface ]->AddTriangle( 33, 34, 35 );

		//create a openGL cube
		updateRegistry();
	}
	
	//======================================================================================================
	void LoadLWO( const std::string& filename, long Flags = 1 ) {
		//Loader for Lightwave Objects (create to make Emm's life a little easier ;) )
		cout << "STARTED:Lightwave Loader v1.0" << endl;
		
		unsigned char* fileData;
		unsigned char buf[ 3 ];
		vector<Face4d> Faces;
		Face4d tempFace;
		vector<Vertex3d> Vertices;
		Vertex3d temp;
		vector<Tag> Tags;
		Tag tempTag;
		/*
		 Here's the same file written in outline form.
		 		 		 		 		 
		 FORM:400:LWO2 already located.
		 
		 DISPLAY HEX DEBUG
		 46  4F  52  4D  00  00  01  90  4C  57  4F  32  54  41  47  53       FORM....LWO2TAGS
		 00  00  00  08  44  65  66  61  75  6C  74  00  4C  41  59  52       ....Default.LAYR
		 00  00  00  12  00  00  00  00  00  00  00  00  00  00  00  00       ................
		 00  00  00  00  00  00  50  4E  54  53  00  00  00  60  BF  80       ......PNTS...`..
		 00  00  BF  80  00  00  BF  80  00  00  BF  80  00  00  3F  80       ..............?.
		 00  00  BF  80  00  00  3F  80  00  00  3F  80  00  00  BF  80       ......?...?.....
		 00  00  3F  80  00  00  BF  80  00  00  BF  80  00  00  BF  80       ..?.............
		 00  00  BF  80  00  00  3F  80  00  00  BF  80  00  00  3F  80       ......?.......?.
		 00  00  3F  80  00  00  3F  80  00  00  3F  80  00  00  3F  80       ..?...?...?...?.
		 00  00  3F  80  00  00  BF  80  00  00  3F  80  00  00  42  42       ..?.......?...BB
		 4F  58  00  00  00  18  BF  80  00  00  BF  80  00  00  BF  80       OX..............
		 00  00  3F  80  00  00  3F  80  00  00  3F  80  00  00  50  4F       ..?...?...?...PO
		 4C  53  00  00  00  64  46  41  43  45  00  03  00  03  00  00       LS...dFACE......
		 00  01  00  03  00  03  00  01  00  02  00  03  00  00  00  04       ................
		 00  01  00  03  00  04  00  05  00  01  00  03  00  02  00  01       ................
		 00  05  00  03  00  02  00  05  00  06  00  03  00  03  00  02       ................
		 00  07  00  03  00  07  00  02  00  06  00  03  00  00  00  03       ................
		 00  04  00  03  00  03  00  07  00  04  00  03  00  04  00  07       ................
		 00  05  00  03  00  07  00  06  00  05  50  54  41  47  00  00       ..........PTAG..
		 00  34  53  55  52  46  00  00  00  00  00  01  00  00  00  02       .4SURF..........
		 00  00  00  03  00  00  00  04  00  00  00  05  00  00  00  06       ................
		 00  00  00  07  00  00  00  08  00  00  00  09  00  00  00  0A       ................
		 00  00  00  0B  00  00  53  55  52  46  00  00  00  2A  44  65       ......SURF...*De
		 66  61  75  6C  74  00  00  00  43  4F  4C  52  00  0E  3F  48       fault...COLR..?H
		 C8  C9  3F  48  C8  C9  3F  48  C8  C9  00  00  44  49  46  46       ..?H..?H....DIFF
		 00  06  3F  80  00  00  00  00  B8  66  D6  70  FF  7F  00  00       ..?......f.p...
		 END OF HEX DEBUG
		 */
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
		//convert data from filename to system
		char* file;
		char* ext;
		strcpy( ext, "lwo" );
		string::traits_type::copy( file, filename.c_str(), filename.length() +1 );
		
		//read in the data
		NSString* NSFilename = convertStringToNSString( filename );
		NSData* lwofile = readDataFromBundle( NSFilename );
		
		if ( lwofile ) {
			//loaded correctly
			fileData = new unsigned char[ lwofile.length ];

			if ( fileData == NULL ) {
				cout << "LoadLWO: Could not allocate buffer " << lwofile.length << " Bytes" << endl;
				exit( 1 );
			}			
			
			[lwofile getBytes:fileData length:lwofile.length];
//			- (void)getBytes:(void *)buffer length:(NSUInteger)length
		}
#else
		//windows loader for lwo files
		FILE* handle;

		handle = fopen( filename.c_str(), "rb" );
		if ( handle == NULL ) {
			//file does not exist
			cout << "LoadLWO:Error loading " << filename << " could not file file." << endl;
			return;									//return null if file doesnt exist
		} else {
			//file exists so start loading it in
			cout << "file exists, loading..." << endl;
			fseek( handle, 0, SEEK_END ); // seek to end of file
			long size = ftell( handle ); // get current file pointer
			fseek( handle, 0, SEEK_SET );
			fileData = new unsigned char[ size ];
//			unsigned char* fileData = (unsigned char*)malloc( size );
			if ( fileData == NULL ) {
				cout << "LoadLWO: Could not allocate buffer " << size << " Bytes" << endl;
				exit( 1 );
			}
			
			cout << "Loading " << size << " Bytes" << endl;
			fread( fileData, 1, size, handle );
			fclose( handle );
		}
#endif
		long tagPos;
		long bufferLocation = 0;
		long fileSize = 0;
		long currentMode = 0;
		long currentValue = 0;
		short gotLayer = FALSE;
		long dataSize = 0;
		long tempValue = 0;
		char tempChar;
		char tempTags[ 255 ];
		long tagCount = 0;
		long currentSurface = 0;
		long LWOVertexCount = 0;
		long LWOTriangleCount = 0;
		
		if ( fileData[ 0 ]=='F' && fileData[ 1 ]=='O' && fileData[ 2 ]=='R' && fileData[ 3 ]=='M' ) {
			if ( fileData[ 8 ] == 'L' && fileData[ 9 ] =='W' && fileData[ 10 ]=='O' && fileData[ 11 ]=='2' ) {
				//FORM 340 LWO2
				cout << "Lightwave object identified, located header for file." << endl;
				buf[ 0 ] = fileData[ 4 ];
				buf[ 1 ] = fileData[ 5 ];
				buf[ 2 ] = fileData[ 6 ];
				buf[ 3 ] = fileData[ 7 ];
				
				cout << "File size " << convertBytesToLong( buf ) << " Bytes, (minus FORM and LWO2 identifiers)" << endl;
				fileSize = convertBytesToLong( buf ) +8;
				
				/*
				 TAGS 8
				 "Default"
				 
				 The TAGS chunk contains an array of strings. Whenever something is identified by name in the file, the ID is often
				 a 0-based index into the TAGS array. The only named element in this file is its single surface, named "Default".
				 */
				cout << "Reading Tags array..." << endl;
				buf[ 0 ] = fileData[ 16 ];
				buf[ 1 ] = fileData[ 17 ];
				buf[ 2 ] = fileData[ 18 ];
				buf[ 3 ] = fileData[ 19 ];
				dataSize = convertBytesToLong( buf );
				cout << "      tags length " << dataSize << endl;
				currentValue = 0;
				for ( tagPos = 20; tagPos < 20+dataSize; tagPos ++ ) {
					tempChar = fileData[ tagPos ];
					if ( tempChar != 0x0 ) {
						//Add the value to the string and move the null end char along
						tempTags[ currentValue ] = tempChar;
						tempTags[ currentValue +1 ] = 0;			//end of string char
						currentValue++;
					} else {
						if ( tempTags[ 0 ] != 0 ) {
							//store old one and reset value
							tempTag.name.assign( tempTags );
							Tags.push_back( tempTag );
							tagCount++;
							cout << "      Tag name:" << tempTag.name << endl;
							tempTags[ 0 ] = 0;
							currentValue = 0;
						}
					}
				}
				cout << "      tag count " << tagCount << endl;
				
				cout << "Reading Lightwave Object Data..." << endl;
				bufferLocation = 20+convertBytesToLong( buf );
				
				while( bufferLocation <fileSize ) {
					//run through buffer
					//No mode is in there so we need to read in the next header chunk of the file
					buf[ 0 ] = fileData[ bufferLocation ];
					buf[ 1 ] = fileData[ bufferLocation +1 ];
					buf[ 2 ] = fileData[ bufferLocation +2 ];
					buf[ 3 ] = fileData[ bufferLocation +3 ];
						
					//Examine the header
					//================================================================================================
					//================================================================================================
					//EXAMINE CHUNKS OF DATA FROM THE ENTIRE THING
					//================================================================================================
					if ( buf[ 0 ] =='L' && buf[ 1 ] =='A' && buf[ 2 ] =='Y' && buf[ 3 ] =='R' ) {
						//LAYER CHUNK
						/*
						 LAYR 18
						 0
						 0
						 0.0  0.0  0.0
						 ""
						 
						 The layer header signals the start of a new layer. All geometry elements that appear in the file after this and
						 before the next LAYR chunk belong to this layer. The layer header contains an index, a flags word, the pivot point
						 of the layer, the layer's name, and the index of the parent layer. This is the first (and only) layer, so its index
						 is 0 and the optional parent index is omitted. The bits in the flags word are also 0, and the layer hasn't been given
						 a name.
						 
						 The pivot point is the origin for rotations in this layer and is expressed in world coordinates. Pivots typically
						 differ from (0, 0, 0) when layers and layer parenting are used to create an object hierarchy.					 
						 */
						cout << "New layer located" << endl;
						if ( gotLayer == TRUE ) {
							//process layer and move to the next one
						}
						currentMode = 1;		//Layer reading mode
						gotLayer = TRUE;
						buf[ 0 ] = fileData[ bufferLocation +4 ];
						buf[ 1 ] = fileData[ bufferLocation +5 ];
						buf[ 2 ] = fileData[ bufferLocation +6 ];
						buf[ 3 ] = fileData[ bufferLocation +7 ];
						currentValue = convertBytesToLong( buf );
						
						cout << "      Layer data length:" << currentValue << endl;
						dataSize = currentValue;
						
						/*
							index( WORD 2Byte )
							flags( WORD 2byte)
							pivot X Y Z (4 Byte each) (12 Bytes)
							Name (None here so size is 0) (WORD 2Byte length)
							*/
						buf[ 2 ] = fileData[ bufferLocation +8 ];
						buf[ 3 ] = fileData[ bufferLocation +9 ];
						cout << "      Index:" << convertBytesToLong( buf ) << endl;
						buf[ 2 ] = fileData[ bufferLocation +10 ];
						buf[ 3 ] = fileData[ bufferLocation +11 ];
						cout << "      Flags:" << convertBytesToLong( buf ) << endl;
						buf[ 0 ] = fileData[ bufferLocation +12 ];
						buf[ 1 ] = fileData[ bufferLocation +13 ];
						buf[ 2 ] = fileData[ bufferLocation +14 ];
						buf[ 3 ] = fileData[ bufferLocation +15 ];
						buf[ 0 ] = 192;
						buf[ 1 ] = 128;
						buf[ 2 ] = 0;
						buf[ 3 ] = 0;
//						cout << "      Pivot:X:" << convertBytesToFloat( buf );
						buf[ 0 ] = fileData[ bufferLocation +16 ];
						buf[ 1 ] = fileData[ bufferLocation +17 ];
						buf[ 2 ] = fileData[ bufferLocation +18 ];
						buf[ 3 ] = fileData[ bufferLocation +19 ];
//						cout << " Y:" << convertBytesToFloat( buf );
						buf[ 0 ] = fileData[ bufferLocation +20 ];
						buf[ 1 ] = fileData[ bufferLocation +21 ];
						buf[ 2 ] = fileData[ bufferLocation +22 ];
						buf[ 3 ] = fileData[ bufferLocation +23 ];
//						cout << " Z:" << convertBytesToFloat( buf ) <<endl;
						buf[ 0 ] = 0;
						buf[ 1 ] = 0;
						buf[ 2 ] = fileData[ bufferLocation +24 ];
						buf[ 3 ] = fileData[ bufferLocation +25 ];
						cout << "      identifier Length:" << convertBytesToLong( buf ) << endl;
						if ( convertBytesToLong( buf )>0 ) {
							cout << "          to be implemented" << endl;
						} else {
							cout << "          No identifier set" << endl;
						}
						
						bufferLocation+=( dataSize +4 );	//add 26+length of LAYR identifier :)
					}
					
					//================================================================================================
					if ( buf[ 0 ] =='P' && buf[ 1 ] =='N' && buf[ 2 ] =='T' && buf[ 3 ] =='S' ) {
						//points data
						/*
						 PNTS 96
						 -0.5  -0.5  -0.5
						 0.5  -0.5  -0.5
						 0.5  -0.5   0.5
						 -0.5  -0.5   0.5
						 -0.5   0.5  -0.5
						 0.5   0.5  -0.5
						 0.5   0.5   0.5
						 -0.5   0.5   0.5
						 
						 The PNTS chunk contains triples of floating-point numbers, the coordinates of a list of points. The numbers are
						 written as IEEE 32-bit floats in network byte order. The IEEE float format is the standard bit pattern used by almost
						 all CPUs and corresponds to the internal representation of the C language float type. In other words, this isn't some
						 bizarre proprietary encoding. You can process these using simple fread and fwrite calls (but don't forget to correct
						 the byte order if necessary).
						 */
						cout << "VERTEX POINTS DATA LOCATED" << endl;
						buf[ 0 ] = fileData[ bufferLocation +4 ];
						buf[ 1 ] = fileData[ bufferLocation +5 ];
						buf[ 2 ] = fileData[ bufferLocation +6 ];
						buf[ 3 ] = fileData[ bufferLocation +7 ];
						currentValue = convertBytesToLong( buf );
						dataSize = currentValue;
						cout << "      Points data length:" << currentValue << endl;
						
						for ( long dataPos = bufferLocation +8; dataPos <( bufferLocation+8 )+dataSize; dataPos += 12 ) {
							//x
							buf[ 0 ] = fileData[ dataPos +0 ];
							buf[ 1 ] = fileData[ dataPos +1 ];
							buf[ 2 ] = fileData[ dataPos +2 ];
							buf[ 3 ] = fileData[ dataPos +3 ];
							temp.X = convertBytesToFloat( buf );
//							printf( "[%x:%x:%x:%x]\n", buf[ 0 ], buf[ 1 ], buf[ 2 ], buf[ 3 ] );

							buf[ 0 ] = fileData[ dataPos +4 ];
							buf[ 1 ] = fileData[ dataPos +5 ];
							buf[ 2 ] = fileData[ dataPos +6 ];
							buf[ 3 ] = fileData[ dataPos +7 ];
							temp.Y = convertBytesToFloat( buf );

							buf[ 0 ] = fileData[ dataPos +8 ];
							buf[ 1 ] = fileData[ dataPos +9 ];
							buf[ 2 ] = fileData[ dataPos +10 ];
							buf[ 3 ] = fileData[ dataPos +11 ];
							temp.Z = convertBytesToFloat( buf );

							Vertices.push_back( temp );
							LWOVertexCount ++;
							printf( "      X:%8.2f  Y:%8.2f  Z:%8.2f\n", temp.X, temp.Y, temp.Z );
//							cout << "      X:" << temp.X << "  Y:" << temp.Y << "  Z:" << temp.Z << endl;
						}
						
						cout << "      Vertex group count " << LWOVertexCount << endl;
						bufferLocation+=( dataSize +4 );	//add data size +header(LAYR) of LAYR identifier :)
					}
					
					//================================================================================================
					if ( buf[ 0 ] =='B' && buf[ 1 ] =='B' && buf[ 2 ] =='O' && buf[ 3 ] =='X' ) {
						//Bounding box
						/*
						 BBOX 24
						 -0.5  -0.5  -0.5
						 0.5   0.5   0.5
						 
						 The bounding box for the layer, just so that readers don't have to scan the PNTS chunk to find the extents.
						 */
						cout << "BOUNDING BOX DATA LOCATED" << endl;
						buf[ 0 ] = fileData[ bufferLocation +4 ];
						buf[ 1 ] = fileData[ bufferLocation +5 ];
						buf[ 2 ] = fileData[ bufferLocation +6 ];
						buf[ 3 ] = fileData[ bufferLocation +7 ];
						currentValue = convertBytesToLong( buf );
						dataSize = currentValue;
						cout << "      Bounding box data length:" << currentValue << endl;
						
						float XW, YW, ZW;
						
						for ( long dataPos = bufferLocation +8; dataPos <( bufferLocation +8 )+dataSize; dataPos+=12 ) {
							//x
							buf[ 0 ] = fileData[ dataPos +0 ];
							buf[ 1 ] = fileData[ dataPos +1 ];
							buf[ 2 ] = fileData[ dataPos +2 ];
							buf[ 3 ] = fileData[ dataPos +3 ];
							XW = convertBytesToFloat( buf );
							//							printf( "[%x:%x:%x:%x]\n", buf[ 0 ], buf[ 1 ], buf[ 2 ], buf[ 3 ] );
							
							buf[ 0 ] = fileData[ dataPos +4 ];
							buf[ 1 ] = fileData[ dataPos +5 ];
							buf[ 2 ] = fileData[ dataPos +6 ];
							buf[ 3 ] = fileData[ dataPos +7 ];
							YW = convertBytesToFloat( buf );
							
							buf[ 0 ] = fileData[ dataPos +8 ];
							buf[ 1 ] = fileData[ dataPos +9 ];
							buf[ 2 ] = fileData[ dataPos +10 ];
							buf[ 3 ] = fileData[ dataPos +11 ];
							ZW = convertBytesToFloat( buf );
							
							printf( "      X:%8.2f  Y:%8.2f  Z:%8.2f\n", XW, YW, ZW );							
						}
						bufferLocation+=( dataSize +4 );	// :)
					}
					
					//================================================================================================
					if ( buf[ 0 ] == 'P' && buf[ 1 ] == 'O' && buf[ 2 ] == 'L' && buf[ 3 ] == 'S' ) {
						/*
						 POLS 64
						 FACE
						 4  0  1  2  3
						 4  0  4  5  1
						 4  1  5  6  2 
						 4  3  2  6  7
						 4  0  3  7  4
						 4  4  7  6  5
						 
						 The POLS chunk contains a list of polygons. A "polygon" in this context is anything that can be described using an
						 ordered list of vertices. A POLS of type FACE contains ordinary polygons, but the POLS type can also be CURV, PTCH,
						 MBAL or BONE, for example.
						 
						 The high 6 bits of the vertex count for each polygon are reserved for flags, which in effect limits the number of
						 vertices per polygon to 1023. Don't forget to mask the high bits when reading the vertex count. The flags are
						 currently only defined for CURVs.
						 
						 The point indexes following the vertex count refer to the points defined in the most recent PNTS chunk. Each index
						 can be a 2-byte or a 4-byte integer. If the high order (first) byte of the index is not 0xFF, the index is 2 bytes
						 long. This allows values up to 65279 to be stored in 2 bytes. If the high order byte is 0xFF, the index is 4 bytes
						 long and its value is in the low three bytes (index & 0x00FFFFFF). The maximum value for 4-byte indexes is 16,777,215
						 (224 - 1). Objects with more than 224 vertices can be stored using multiple pairs of PNTS and POLS chunks.
						 
						 The cube has 6 square faces each defined by 4 vertices. LightWave polygons are single-sided by default
						 (double-sidedness is a possible surface property). The vertices are listed in clockwise order as viewed from the
						 visible side, starting with a convex vertex. (The normal is defined as the cross product of the first and last edges.)
						 */
						cout << "POLYGON DATA LOCATED" << endl;
						buf[ 0 ] = fileData[ bufferLocation +4 ];
						buf[ 1 ] = fileData[ bufferLocation +5 ];
						buf[ 2 ] = fileData[ bufferLocation +6 ];
						buf[ 3 ] = fileData[ bufferLocation +7 ];
						currentValue = convertBytesToLong( buf );
						dataSize = currentValue;
						cout << "      Polygon data length:" << currentValue << endl;
						buf[ 0 ] = fileData[ bufferLocation +8 ];
						buf[ 1 ] = fileData[ bufferLocation +9 ];
						buf[ 2 ] = fileData[ bufferLocation +10 ];
						buf[ 3 ] = fileData[ bufferLocation +11 ];						
						if ( buf[ 0 ] == 'F' && buf[ 1 ] == 'A' && buf[ 2 ] == 'C' && buf[ 3 ] == 'E' ) {
							cout << "      Face data identified" << endl;
						}
						long VP = bufferLocation +12;
						do {
							// vertex count (number of corners)
							buf[ 0 ] = 0;
							buf[ 1 ] = 0;
							buf[ 2 ] = fileData[ VP ];
							buf[ 3 ] = fileData[ VP+1 ];
							tempFace.VertexCount = convertBytesToLong( buf );
							cout << "      Face data " << tempFace.VertexCount << endl;

							if ( tempFace.VertexCount == 3 ) { 
								cout << "      Face data triangle" << endl;
								buf[ 0 ] = 0;
								buf[ 1 ] = 0;
								buf[ 2 ] = fileData[ VP+2 ];
								buf[ 3 ] = fileData[ VP+3 ];
								tempFace.Vertex[ 0 ] = convertBytesToLong( buf );
								buf[ 0 ] = 0;
								buf[ 1 ] = 0;
								buf[ 2 ] = fileData[ VP+4 ];
								buf[ 3 ] = fileData[ VP+5 ];
								tempFace.Vertex[ 1 ] = convertBytesToLong( buf );
								buf[ 0 ] = 0;
								buf[ 1 ] = 0;
								buf[ 2 ] = fileData[ VP+6 ];
								buf[ 3 ] = fileData[ VP+7 ];
								tempFace.Vertex[ 2 ] = convertBytesToLong( buf );
								cout << "      Vertex data V1:" << tempFace.Vertex[ 0 ] << " V2:" << tempFace.Vertex[ 1 ] << " V3:" << tempFace.Vertex[ 2 ] << endl;
								LWOTriangleCount++;
								VP+=7;
							} else {
								cout << "      Face data quad" << endl;
								buf[ 0 ] = 0;
								buf[ 1 ] = 0;
								buf[ 2 ] = fileData[ VP+2 ];
								buf[ 3 ] = fileData[ VP+3 ];
								tempFace.Vertex[ 0 ] = convertBytesToLong( buf );
								buf[ 0 ] = 0;
								buf[ 1 ] = 0;
								buf[ 2 ] = fileData[ VP+4 ];
								buf[ 3 ] = fileData[ VP+5 ];
								tempFace.Vertex[ 1 ] = convertBytesToLong( buf );
								buf[ 0 ] = 0;
								buf[ 1 ] = 0;
								buf[ 2 ] = fileData[ VP+6 ];
								buf[ 3 ] = fileData[ VP+7 ];
								tempFace.Vertex[ 2 ] = convertBytesToLong( buf );
								buf[ 0 ] = 0;
								buf[ 1 ] = 0;
								buf[ 2 ] = fileData[ VP+8 ];
								buf[ 3 ] = fileData[ VP+9 ];
								tempFace.Vertex[ 3 ] = convertBytesToLong( buf );
								cout << "      Vertex data V1:" << tempFace.Vertex[ 0 ] << " V2:" << tempFace.Vertex[ 1 ] << " V3:" << tempFace.Vertex[ 2 ] << " V4:" << tempFace.Vertex[ 3 ] << endl;
								LWOTriangleCount++;
								VP+=9;
							}
							Faces.push_back( tempFace );		//add faces to the vector list
							VP++;
						} while( VP < ( bufferLocation +8 )+dataSize );
						
						cout << "      Triangle count " << LWOTriangleCount << endl;
						bufferLocation+=( dataSize +4 );	// :)
					}

					//================================================================================================
					if ( buf[ 0 ] == 'P' && buf[ 1 ] == 'T' && buf[ 2 ] == 'A' && buf[ 3 ] == 'G' ) {
						/*
						 PTAG 28
						 SURF
						 0  0
						 1  0
						 2  0
						 3  0
						 4  0
						 5  0
						 
						 The PTAG chunk associates tags with polygons. In this case, it identifies which surface is assigned to each polygon.
						 The first number in each pair is a 0-based index into the most recent POLS chunk, and the second is a 0-based index
						 into the TAGS chunk.
						 */
						cout << "POLYGON SURFACE INDEX DATA LOCATED" << endl;
						buf[ 0 ] = fileData[ bufferLocation +4 ];
						buf[ 1 ] = fileData[ bufferLocation +5 ];
						buf[ 2 ] = fileData[ bufferLocation +6 ];
						buf[ 3 ] = fileData[ bufferLocation +7 ];
						currentValue = convertBytesToLong( buf );
						dataSize = currentValue;
						cout << "      Polygon surface index data length:" << currentValue << endl;
						
						buf[ 0 ] = fileData[ bufferLocation +8 ];
						buf[ 1 ] = fileData[ bufferLocation +9 ];
						buf[ 2 ] = fileData[ bufferLocation +10 ];
						buf[ 3 ] = fileData[ bufferLocation +11 ];
						if ( buf[ 0 ] == 'S' && buf[ 1 ] == 'U' && buf[ 2 ] == 'R' && buf[ 3 ] == 'F' ) {
							//Surface type Identified
							cout << "      Surface information located" << endl;
						}
						
						for ( long dataPos = bufferLocation +12; dataPos <( bufferLocation +8 )+dataSize; dataPos+=4 ) {
							//read in the relevant data
							buf[ 0 ] = 0;
							buf[ 1 ] = 0;
							buf[ 2 ] = fileData[ dataPos ];
							buf[ 3 ] = fileData[ dataPos +1 ];
							tempValue =  convertBytesToLong( buf );
							buf[ 0 ] = 0;
							buf[ 1 ] = 0;
							buf[ 2 ] = fileData[ dataPos +2 ];
							buf[ 3 ] = fileData[ dataPos +3 ];
							Faces[ tempValue ].SurfaceID = convertBytesToLong( buf );
							
							cout << "      Vertex ID:" << tempValue << "  Surface ID:" << convertBytesToLong( buf ) << endl;
						}

						bufferLocation+=( dataSize +4 );	// :)
					}
					
					//================================================================================================
					if ( buf[ 0 ] == 'S' && buf[ 1 ] == 'U' && buf[ 2 ] == 'R' && buf[ 3 ] == 'F' ) {
						/*
						 SURF 42
						 "Default"
						 ""
						 COLR 14
						 0.78431  0.78431  0.78431
						 0
						 DIFF 6
						 1.0
						 0
						 
						 The description of each surface is stored in a SURF chunk. The only items guaranteed to be in a SURF chunk are the
						 names of the surface and of its parent. The parent name is often empty, but if it's not, any surface parameters not
						 defined in the SURF can be inherited from the parent's SURF. When there's no parent, undefined parameters are
						 assigned default values. ("Default" is just the default name. If you aren't concerned about confusing people, you're
						 free to give non-default values to a surface with this name.)
						 
						 Following the name fields is a collection of subchunks, each of which defines a property of the surface. Like IFF
						 chunks, SURF subchunks start with a 4-byte ID followed by a chunk size, but the size is 2 bytes in length rather
						 than 4. Although subchunks tend to be quite small, SURFs may contain a large number of them, as we'll see later.
						 
						 The 0 at the end of the COLR and DIFF subchunks indicates that these surface attributes are not enveloped (don't
						 vary over time). We'll change that later, too.
						 
						 00  00  00  0B  00  00  53  55  52  46  00  00  00  40  45  64       ......SURF...@Ed
						 43  75  62  65  00  00  00  00  43  4F  4C  52  00  0E  3F  48       Cube....COLR..?H
						 C8  C9  00  00  00  00  00  00  00  00  00  00  44  49  46  46       ............DIFF
						 00  06  3F  80  00  00  00  00  53  50  45  43  00  06  3E  D4       ..?.....SPEC..>.
						 */
						cout << "SURFACE DATA LOCATED" << endl;
						buf[ 0 ] = fileData[ bufferLocation +4 ];
						buf[ 1 ] = fileData[ bufferLocation +5 ];
						buf[ 2 ] = fileData[ bufferLocation +6 ];
						buf[ 3 ] = fileData[ bufferLocation +7 ];
						currentValue = convertBytesToLong( buf );
						dataSize = currentValue;
						cout << "      Surface data length:" << currentValue << endl;
						
						for ( currentSurface = 0; currentSurface<tagCount; currentSurface++ ) {
							bufferLocation += ( Tags[ currentSurface ].name.size() +12 );		//MOVE pointer forwards
							
							buf[ 0 ] = fileData[ bufferLocation ];
							buf[ 1 ] = fileData[ bufferLocation +1 ];
							buf[ 2 ] = fileData[ bufferLocation +2 ];
							buf[ 3 ] = fileData[ bufferLocation +3 ];
							if ( buf[ 0 ] == 'C' && buf[ 1 ] == 'O' && buf[ 2 ] == 'L' && buf[ 3 ] == 'R' ) {
								/*
								 COLR { base-color[COL12 (3Floats)], envelope[VX] }
								 The base color of the surface, which is the color that lies under all the other
								 texturing attributes.
								 
								 VX ::= index[U2] | (index + 0xFF000000)[U4]
								 This is an index into an array of items (points or polygons), or a collection of items 
								 each uniquely identified by an integer (clips or envelopes). A VX is written as a variable
								 length 2- or 4-byte element. If the index value is less than 65,280 (0xFF00), then the index
								 is written as an unsigned two-byte integer. Otherwise the index is written as an unsigned 
								 four byte integer with bits 24-31 set. When reading an index, if the first byte encountered
								 is 255 (0xFF), then the four-byte form is being used and the first byte should be discarded
								 or masked out.
								 */
								cout << "      Surface colour information identified." << endl;
								buf[ 0 ] = fileData[ bufferLocation +4 ];
								buf[ 1 ] = fileData[ bufferLocation +5 ];
								buf[ 2 ] = fileData[ bufferLocation +6 ];
								buf[ 3 ] = fileData[ bufferLocation +7 ];
								Tags[ currentSurface ].Red = convertBytesToFloat( buf );
								buf[ 0 ] = fileData[ bufferLocation +8 ];
								buf[ 1 ] = fileData[ bufferLocation +9 ];
								buf[ 2 ] = fileData[ bufferLocation +10 ];
								buf[ 3 ] = fileData[ bufferLocation +11 ];
								Tags[ currentSurface ].Green = convertBytesToFloat( buf );
								buf[ 0 ] = fileData[ bufferLocation +12 ];
								buf[ 1 ] = fileData[ bufferLocation +13 ];
								buf[ 2 ] = fileData[ bufferLocation +14 ];
								buf[ 3 ] = fileData[ bufferLocation +15 ];
								Tags[ currentSurface ].Blue = convertBytesToFloat( buf );
								buf[ 0 ] = fileData[ bufferLocation +12 ];
								buf[ 1 ] = fileData[ bufferLocation +13 ];
								buf[ 2 ] = fileData[ bufferLocation +14 ];
								buf[ 3 ] = fileData[ bufferLocation +15 ];
								Tags[ currentSurface ].colourEnvelope = convertBytesToLong( buf );
								
								printf( "      Red:%3.2f ", Tags[ currentSurface ].Red );
								printf( "Green:%3.2f ", Tags[ currentSurface ].Green );
								printf( "Blue:%3.2f\n", Tags[ currentSurface ].Blue );
								printf( "      Envelope %li\n", Tags[ currentSurface ].colourEnvelope );
								
								bufferLocation += 20;
							}
							
							buf[ 0 ] = fileData[ bufferLocation ];
							buf[ 1 ] = fileData[ bufferLocation +1 ];
							buf[ 2 ] = fileData[ bufferLocation +2 ];
							buf[ 3 ] = fileData[ bufferLocation +3 ];
							if ( buf[ 0 ] == 'D' && buf[ 1 ] == 'I' && buf[ 2 ] == 'F' && buf[ 3 ] == 'F' ) {
								buf[ 0 ] = fileData[ bufferLocation +4 ];
								buf[ 1 ] = fileData[ bufferLocation +5 ];
								buf[ 2 ] = fileData[ bufferLocation +6 ];
								buf[ 3 ] = fileData[ bufferLocation +7 ];
								cout << "      Diffusion " << convertBytesToFloat( buf ) << endl;
								bufferLocation += 12;
							}
							buf[ 0 ] = fileData[ bufferLocation ];
							buf[ 1 ] = fileData[ bufferLocation +1 ];
							buf[ 2 ] = fileData[ bufferLocation +2 ];
							buf[ 3 ] = fileData[ bufferLocation +3 ];
							//printf( "%c%c%c%c\n", buf[ 0 ], buf[ 1 ], buf[ 2 ], buf[ 3 ] );
							if ( buf[ 0 ] == 'S' && buf[ 1 ] == 'P' && buf[ 2 ] == 'E' && buf[ 3 ] == 'C' ) {
								buf[ 0 ] = fileData[ bufferLocation +4 ];
								buf[ 1 ] = fileData[ bufferLocation +5 ];
								buf[ 2 ] = fileData[ bufferLocation +6 ];
								buf[ 3 ] = fileData[ bufferLocation +7 ];
								cout << "      Specularity/Shininess " << convertBytesToFloat( buf ) << endl;
								bufferLocation += 12;								
							}
							
//							for ( long dataPos = bufferLocation +8; dataPos <( bufferLocation +8 )+dataSize; dataPos++ ) {								
//								printf( "%c", fileData[ dataPos ] );
//							}
							bufferLocation += ( dataSize +4 );
						}
						printf( "\n" );
//						bufferLocation+=( dataSize +4 );	// :)
					}
					bufferLocation++;
				}
			}
		}
		
		cout << "END OF LWO LOADER" << endl;
		
		cout << "Converting LWO mesh data to Tombstone Entity data..." << endl;
		long tempSurface;
		long vert1, vert2, vert3;
		
		CreateEntity();
		
		tempSurface = CreateSurface();		
		for ( long LWOTPos = 0; LWOTPos < LWOTriangleCount; LWOTPos++ ) {
			if ( Faces[ LWOTPos ].VertexCount == 3 ) {
				//Triangle
				vert1 = Surface[ tempSurface ]->AddVertex( Vertices[ Faces[ LWOTPos ].Vertex[ 0 ] ].X, Vertices[ Faces[ LWOTPos ].Vertex[ 0 ] ].Y, Vertices[ Faces[ LWOTPos ].Vertex[ 0 ] ].Z, 0.0, 0.0 );
				vert2 = Surface[ tempSurface ]->AddVertex( Vertices[ Faces[ LWOTPos ].Vertex[ 1 ] ].X, Vertices[ Faces[ LWOTPos ].Vertex[ 1 ] ].Y, Vertices[ Faces[ LWOTPos ].Vertex[ 1 ] ].Z, 0.0, 0.0 );
				vert3 = Surface[ tempSurface ]->AddVertex( Vertices[ Faces[ LWOTPos ].Vertex[ 2 ] ].X, Vertices[ Faces[ LWOTPos ].Vertex[ 2 ] ].Y, Vertices[ Faces[ LWOTPos ].Vertex[ 2 ] ].Z, 0.0, 0.0 );
				Surface[ tempSurface ]->AddTriangle( vert1, vert2, vert3 );
				cout << "Vert1:" << vert1 << " Vert2:" << vert2 << " Vert3:" << vert3 << endl;
			} else {
				//Quad (needs converting to triangles)
			}
		}
		updateRegistry();
		
		//EntityDebug.DisplayHex( fileData, 0, 408 );
		
		//now we got the data we need to process it
		delete[] fileData;
	}
	
	//======================================================================================================
	long loadB3d( const std::string& filename, long Flags = 1 ) {
		//this will load the Blitz3d model
		/*
		 Flags are simply the same as Blitz3d :)
		 */
		return( 0 );										//if null then there will be NO entry loaded
	}
	
	//======================================================================================================
	void PositionEntity( GLfloat x, GLfloat y, GLfloat z ) {
		X = x;
		Y = y;
		Z = z;
		updateRegistry();
	}

	//======================================================================================================
	void TurnEntity( GLfloat pitch, GLfloat yaw, GLfloat roll ) {
		Xr += pitch;
		if ( Xr >360.0 ) Xr -=360;
		if ( Xr <0.0 ) Xr +=360;
		Yr += yaw;
		if ( Yr >360.0 ) Yr -=360;
		if ( Yr <0.0 ) Yr +=360;
		Zr += roll;
		if ( Zr >360.0 ) Zr -=360;
		if ( Zr <0.0 ) Zr +=360;
		updateRegistry();
	}
	
	//======================================================================================================
	void RotateEntity( GLfloat pitch, GLfloat yaw, GLfloat roll ) {
		Xr = pitch;
		Yr = yaw;
		Zr = roll;
		updateRegistry();
	}
};
