/*
 *  TSound.h
 *  Treasure Caves
 *
 *  Created by EdzUp on 25/07/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

//This will handle the OpenAL sound
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
	//import iPhone system into sound engine
	#import "TombstoneiPhoneFunctions.h"
	#import <AudioToolbox/AudioServices.h>
	#import "AVFoundation/AVFoundation.h"
	#import <UIKit/UIKit.h>
#endif
#import <OpenAL/al.h>
#import <OpenAL/alc.h>

class TombstoneSoundClass {
public:
	//contexts for OpenAL system
	ALCcontext* openALContext;
	ALCdevice* openALDevice;
	
	void StartOpenALSoundSystem( void ) {
		//Starting the system
		openALDevice = alcOpenDevice( NULL );			//Select the 'Preferred device'
	
		if ( openALDevice ) {
			//use the device to make a context
			openALContext = alcCreateContext( openALDevice, NULL );
		
			//MAKE SURE ITS THE ACTIVE ONE
			alcMakeContextCurrent( openALContext );
		}
	}

	void ShutdownOpenALSoundSystem( void ) {
		// delete the sources
		//DELETE ALL SOUND SOURCES
	
		//DELETE ALL SOUND BUFFERS

		// destroy the context
		alcDestroyContext( openALContext );
		// close the device
		alcCloseDevice(openALDevice );
	}
};

TombstoneSoundClass TombstoneSound;

//OpenAL Sound system
class TSoundClass {
public:
	ALuint Sound;								//this is the sound
	ALuint Source;								//this is the player source
		
	//---------------------------------------------------------------------------------------------
	void loadSound( const std::string& filename, short loops = FALSE ) {
		//this will be used to Load an OpenAL sound
		#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
			//iPhone implementation
			NSString* filelocation = iPhoneFilename( filename );
			AudioFileID outAFID;
			NSURL* afUrl = [ NSURL fileURLWithPath:filelocation];
			#if TARGET_OS_IPHONE
				OSStatus result = AudioFileOpenURL( (CFURLRef)afUrl, kAudioFileReadPermission, 0, &outAFID );
			#else
				OSStatus result = AudioFileOpenURL( (CFURLRef)afUrl, fsRdPerm, 0, &outAFID );
			#endif

			result = noErr;
			UInt32 fileSize = 0;
			UInt32 thePropSize = sizeof( UInt64 );
			result = AudioFileGetProperty( outAFID, kAudioFilePropertyAudioDataByteCount, &thePropSize, &fileSize );
			if(result != 0) NSLog( @"cannot find file size" );
		
			unsigned char* outData = new unsigned char [ fileSize ];
			result = AudioFileReadBytes( outAFID, false, 0, &fileSize, outData );
			AudioFileClose( outAFID );		//Close the file
		
			if ( result != 0 ) cout << "ERROR:cannot load effect " << filename << endl;		
		#else
			//Everyone else
			long fileSize = 0;
			unsigned char* outData = new unsigned char [ fileSize ];
		#endif

		alGenBuffers( 1, &Sound );		//generate the sound buffer
		alBufferData( Sound, AL_FORMAT_STEREO16, outData, fileSize, 44100 );
		alGenSources( 1, &Source );		//grab the source ID from openAL
		
		// attach the buffer to the source
		alSourcei( Source, AL_BUFFER, Sound );
		// set some basic source prefs
		alSourcef( Source, AL_PITCH, 1.0f );
		alSourcef( Source, AL_GAIN, 1.0f );
		if ( loops == TRUE ) alSourcei( Source, AL_LOOPING, AL_TRUE );		
		
		//universal code
		if ( outData ) {
			delete [] outData;
			outData = NULL;
		}		
	}

	//---------------------------------------------------------------------------------------------
	void playSound( void ) {
		//plays the OpenAL sound
		alSourcePlay( Source );
	}

	//---------------------------------------------------------------------------------------------
	void stopSound( void ) {
		//stop a OpenAL Sound
		alSourceStop( Source );
	}

	//---------------------------------------------------------------------------------------------
	void freeSound( void ) {
		//this will delete the sound and release the buffer from memory
		alDeleteSources( 1, &Source );
		alDeleteBuffers( 1, &Sound );
	}
};

typedef TSoundClass TSound;

#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
//NO LOADING OF APPLE STUFF ON OTHER PLATFORMS
class TAppleSoundClass {
public:
	AVAudioPlayer *sound;
	float soundVolume;
	
	//============================================================================================================================
	void playSound( float volume ) {
		//play a sound at given volume
		sound.volume = volume;
		soundVolume = volume;
		[sound play];
	}
	
	//============================================================================================================================
	void loadSound( const std::string& filename, long loopCount ) {
//	AVAudioPlayer *loadSound( NSString *filename, long loopCount ) {
		/*
		 NSURL *url = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/crowd.wav", [[NSBundle mainBundle] resourcePath]]];
		 NSError *error;
		 soundCrowd = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:&error];
		 soundCrowd.numberOfLoops = 0;*/
		NSString *NSfilename = iPhoneFilename( filename.c_str() );
		NSURL *lsurl = [NSURL fileURLWithPath:[NSString stringWithFormat:NSfilename, [[NSBundle mainBundle] resourcePath]]];
		NSError *lserror;
		
		sound = [[AVAudioPlayer alloc] initWithContentsOfURL:lsurl error:&lserror];
		sound.numberOfLoops = loopCount;
		[sound prepareToPlay];
	}

	//============================================================================================================================
	void setVolume( float volume ) {
		//change the volume
		sound.volume = volume;
		soundVolume = volume;
	}

	//============================================================================================================================
	void freeSound( void ) {
		[sound release];
	}
};

typedef TAppleSoundClass TAppleSound;
#endif
