/*
 *  TombstoneSurface.h
 *  Tombstone Demo
 *
 *  Created by EdzUp on 09/06/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */
#if defined( __APPLE__ )					//Apple platform uses OpenGL ES
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
#import <UIKit/UIKit.h>
#import <OpenGLES/ES1/gl.h>
#import "TombstoneiPhoneFunctions.h"
#else
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <GLUT/GLUT.h>
#import "TombstoneOSXFunctions.h"
#endif
#endif

#if defined( __LINUX__ )
#include <GL/glut.h>   				 	// Header File For The GLUT Library
#include <GL/gl.h>						// Header File For The OpenGL32 Library
#include <GL/glu.h>						// Header File For The GLu32 Library
#include <unistd.h>						// Header File For sleeping.
#import "TombstoneLinuxFunctions.h"		//all linux declarations for the entire engine
#endif

#if defined( __WINDOWS__ )
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library#import <windows.h> on windows platform import the Windows headers
#import "TombstoneWindowsFunctions.h"	//windows functions
#endif

#include <iostream>
#include <string>
#include <list>
#include <vector>
#import "TombstoneGlu.h"				//required for ALL MATH stuff
#import "TTexture.h"

#define SURFACEFX_NONE						0
#define SURFACEFX_FULLBRIGHT				1
#define SURFACEFX_VERTEXCOLOUR				2
#define SURFACEFX_FLATSHADED				4
#define	SURFACEFX_DISABLEFOG				8
#define SURFACEFX_DISABLEBACKFACECULLING	16
#define SURFACEFX_FORCEALPHABLENDING		32

using namespace std;

class TSurfaceClass;
typedef TSurfaceClass TSurface;

class TSurfaceClass {
public:
	long	Id;								//Id of this surface
	float	ShineValue;						//how shiny this surface is
	TTexture	textureID[ 8 ];					//maximum of eight texture layers
	vector <GLfloat> verts;					//the vertices of the mesh ( X, Y, Z )
	vector <GLubyte> triangleIndex;			//GLUint as we require more than 255 vertices PER mesh (this gives us 65535)
	vector <GLfloat> tverts;				//Texture vertices
	vector <GLfloat> normals;				//Entity normals
	vector <GLfloat> vertexColours;			//R G B of vertices
	long	vertexPoints;					//how many vertex points it has
	long	texturePoints;					//how many texture points it has
	
	short	fxFullBright;
	short	fxVertexColour;
	short	fxFlatShaded;
	short	fxFogDisabled;
	short	fxBackfaceDisabled;
	short	fxAlphaBlending;

	//======================================================================================================
	void VertexColour( long vertexID, GLfloat red, GLfloat green, GLfloat blue ) {
		//*4 as RGBA is used
		vertexColours[ vertexID *4 ] = ( 1.0 /255.0 ) *red;
		vertexColours[ ( vertexID *4 ) +1 ] = ( 1.0 /255.0 ) *green;
		vertexColours[ ( vertexID *4 ) +2 ] = ( 1.0 /255.0 ) *blue;
	}

	//======================================================================================================
	void VertexAlpha( long vertexID, GLfloat alpha ) {
		//change the alpha property of the selected vertices
		vertexColours[ ( vertexID *4 ) +3 ] = alpha;
	}

	//======================================================================================================
	long AddVertex( GLfloat x, GLfloat y, GLfloat z, GLfloat u = 0.0, GLfloat v = 0.0, GLfloat w = 50.0 ) {
		//Add the vertices to the vector array
		/*
		 The optional u, v and w parameters allow you to specify texture coordinates for a vertex, which will determine how
		 any triangle created using those vertices will be texture mapped. The u, v and w parameters specified will take
		 effect on both texture coordinate sets (0 and 1). This works on the following basis: 
		 
		 The top left of an image has the uv coordinates 0,0. 
		 The top right has coordinates 1,0
		 The bottom right is 1,1.
		 The bottom left 0,1. 
		 */
		verts.push_back( x );
		verts.push_back( y );
		verts.push_back( z );
		
		vertexColours.push_back( 1.0 );	//R
		vertexColours.push_back( 1.0 );	//G
		vertexColours.push_back( 1.0 );	//B
		vertexColours.push_back( 1.0 );	//Alpha
		
		tverts.push_back( u );
		tverts.push_back( v );
		if ( w != 50.0 ) tverts.push_back( w );
		
		vertexPoints++;
		
		return( vertexPoints -1 );		//return last Vertex entry
	}
	
	//======================================================================================================
	void SurfaceFX( long FX ) {
		/*
		fx -
		0: nothing (default)
		1: full-bright
		2: use vertex colors instead of brush color
		4: flatshaded
		8: disable fog
		16: disable backface culling
		32: force alpha-blending
		 */
		long FXTemp = FX;
		
		//set it to 0 (nothing default)
		fxAlphaBlending = FALSE;
		fxBackfaceDisabled = FALSE;
		fxFogDisabled = FALSE;
		fxFlatShaded = FALSE;
		fxVertexColour = FALSE;
		fxFullBright = FALSE;
		
		cout << "Surface FX:" << endl;
		//run through helper system to check out different values
		if ( FXTemp-32 >-1 ) {
			//Alpha blending check
			cout << "-Alpha Blending" << endl;
			FXTemp -= 32;
			fxAlphaBlending = TRUE;
		}
		
		if ( FXTemp-16 >-1 ) {
			//Backface Culling
			cout << "-Backface Culling" << endl;
			FXTemp -= 16;
			fxBackfaceDisabled = TRUE;
		}
		
		if ( FXTemp-8>-1 ) {
			//Fog
			cout << "-Fog" << endl;
			FXTemp -= 8;
			fxFogDisabled = TRUE;
		}
		
		if ( FXTemp-4>-1 ) {
			//Flat shaded
			cout << "-Flat shaded" << endl;
			FXTemp -= 4;
			fxFlatShaded = TRUE;
		}
		
		if ( FXTemp-2>-1 ) {
			//Vertex Colour
			cout << "-Vertex colour instead of GL Lighting" << endl;
			FXTemp -= 2;
			fxVertexColour = TRUE;
		}
		
		if ( FXTemp-1>-1 ) {
			//Full Bright
			cout << "-Full bright" << endl;
			fxFullBright = TRUE;
		}
	}
	
	//======================================================================================================
	void AddTriangle( GLuint v1, GLuint v2, GLuint v3 ) {
		//adds a triangle to the triangle index array
		triangleIndex.push_back( v1 );
		triangleIndex.push_back( v2 );
		triangleIndex.push_back( v3 );
	}
	
	//======================================================================================================
	void Shininess( GLfloat shine ) {
		ShineValue = shine;
	}
	
	//======================================================================================================
	void Specularity( GLfloat shine ) {
		ShineValue = shine;
	}
	/*
	//======================================================================================================
	void updateTextureEntry( long entry, GLuint newTexture ) {
		//required for adjustment system
		textureID[ entry ] = newTexture;
	}	
	*/
	//======================================================================================================
	void TextureSurface( const GLuint texture, int texturePosition ) {
		textureID[ texturePosition ].Tname = texture;
	}
};
