/*
 *  TText.h
 *  Treasure Caves
 *
 *  Created by EdzUp on 13/07/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

//this will hold all text declarations for the engine requirements
#import "TSprite.h"

class TTextClass {
public:
	string	TextBuffer;
	int		characterHeight;
	int		characterWidth;
	TSprite	*textImage;
	GLfloat textAngle;					//used to draw text in an angle
	int		characterCount;
	
	//======================================================================================================
	void deleteFont( void ) {
		//remove the font
		for ( int i = 0; i< characterCount; i++ ) {
			if ( textImage[ i ].name.Tname != 0x0 ) {
				textImage[ i ].FreeSprite();
			}
		}
		
		delete []textImage;
	}

	//======================================================================================================
	void SetupTextSystem( const int width, const int height, const int charCount, const std::string& textbuffer ) {		
		characterHeight = height;
		characterWidth = width;
		TextBuffer = textbuffer;
		textImage = new TSprite[ charCount ];
		characterCount = charCount;
	}
	
	//======================================================================================================
	void SetTextAngle( const GLfloat Angle ) {
		//this will be used to set the text angle for all text effects
		textAngle = Angle;
	}
	
	//======================================================================================================
	void DrawText( const std::string& text, GLfloat x, GLfloat y, short centerX = FALSE, short centerY = FALSE, short rotateText = FALSE ) {
		//this function will be used to draw ALL text for the system
		GLfloat tX;
		GLfloat tY;
		int currentChar;
		int stringLength;
		int temp;
		string	charToFind;
		GLfloat DTXinc;
		GLfloat DTYinc;
		GLfloat rText;
		
		rText = imageDrawAngle;
		
		if ( rotateText == TRUE ) {
			//rotate the text to a given angle dependent on the required angle
			DTXinc = sin( ( textAngle *M_PI )/180 ) *characterWidth;
			DTYinc = cos( ( textAngle *M_PI )/180 ) *characterWidth;
			setSpriteRotation( ( 0-textAngle ) +90.0 );
		} else {
			DTXinc = ( characterWidth );
			DTYinc = 0;
		}
		
		tX = x;
		tY = y;
		if ( rotateText == FALSE ) {
			if ( centerX == TRUE ) {
				//center the text
				tX = tX - ( ( text.size() *(characterWidth *1.5 ) ) /2 );
			}
			if ( centerY == TRUE ) {
				//center the text
				tY = tY - cos( characterHeight /2 );
			}
		} else {
			if ( centerX == TRUE ) {
				//center rotated text
				tX = tX - ( ( text.size() *DTXinc ) /2 );
				tY = tY - ( ( text.size() *DTYinc ) /2 );
			}
		}
		stringLength = text.size();
		
		for ( currentChar = 0; currentChar <stringLength; currentChar++ ){
			charToFind = Mid( text, currentChar, 1 );
			temp = InStr( TextBuffer, charToFind, 0 ); //[ currentChar ]	//check in the buffer to see if its there
			if ( temp != -1 ) {
				//we got one :)
				textImage[ temp ].PasteSprite( tX, tY );
			}
			tX = tX +(DTXinc *1.5);//sin( characterWidth*1.5 );
			tY = tY +(DTYinc *1.5);//cos( characterHeight*1.5 );
		}
		if ( rotateText == TRUE ) {
			//restore the angle back to the system for usage
			setSpriteRotation( rText );
		}
	}	
};

typedef TTextClass TText;
