/*
 *  TombstoneAppleGameCenter.h
 *  Treasure Caves
 *
 *  Created by EdzUp on 28/09/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

/*
 This will include all the Apple Game Center code so we can use Apple Game Center in our applications
*/

#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
//Only available on iPhone

BOOL gameCenterOk;

BOOL isGameCenterAvailable();

BOOL checkAppleGameCenterInstalled( void ) {
	gameCenterOk = isGameCenterAvailable();
	
	return( gameCenterOk );
}

BOOL isGameCenterAvailable() {
	// Check for presence of GKLocalPlayer API. 
	Class gcClass = (NSClassFromString(@"GKLocalPlayer"));
	
	// The device must be running running iOS 4.1 or later. 
	NSString *reqSysVer = @"4.1"; 
	NSString *currSysVer = [[UIDevice currentDevice] systemVersion]; 
	BOOL osVersionSupported = ([currSysVer compare:reqSysVer options:NSNumericSearch] != NSOrderedAscending); 
	
	return (gcClass && osVersionSupported);
}

void authenticationChanged( void ) {
	if ([GKLocalPlayer localPlayer].isAuthenticated) {
		// Insert code here to handle a successful authentication.
	} else {
		// Insert code here to clean up any outstanding Game Center-related classes.
	}
}

void AppleGameCenterRegisterAuthenticationNotification( void ) {
	//setup handler to tell app if game center user changes
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc addObserver: nil
		   selector:@selector( authenticationChanged )
			   name:GKPlayerAuthenticationDidChangeNotificationName
			 object:nil];
}

void AppleGameCenterAuthentication( void ) {
	//this will authenticate the player
	[[GKLocalPlayer localPlayer] authenticateWithCompletionHandler:^(NSError *error) {
		if ( error == nil ) {
			//insert code for successful authentication
			AppleGameCenterRegisterAuthenticationNotification();
		} else {
			//report error to player
		}
	}];
}

void AppleGameCenterCheckAuthentication( void ) {
	//check to see if player authentication has changed
	if ([GKLocalPlayer localPlayer].isAuthenticated) {
		// Insert code here to handle a successful authentication.
	} else {
		// Insert code here to clean up any outstanding Game Center-related classes.
	}
}
#endif
