/*
 *  EdzUpFileSystem.h
 *  Created by EdzUp on 23/09/1997.
 *  Copyright 1997 EdzUp. All rights reserved.
 */
#include <string>
#include <fstream>
#include <vector>
#include <iostream>

using namespace std;

long minimumFileSize;							//this is the minimum file size for a file on this file system

ifstream inFile;

//-----------------------------------------------------------------------------------------------------------------
void OpenFile( const char* filename ) {
	inFile.open( filename );
	
	if(!inFile) { // file couldn't be opened
		cerr << "Error: file could not be opened : " << filename <<endl;
		exit(1);
	}
}

//-----------------------------------------------------------------------------------------------------------------
void CloseFile( void ) {
	inFile.close();
}

//-----------------------------------------------------------------------------------------------------------------
string ReadLine( void ) {
	//this will read a line from a file then return the text
	string temp;
	
	getline( inFile, temp );

	return( temp );
}

//-----------------------------------------------------------------------------------------------------------------
short deleteFile( const char* filename ) {
	//this function deletes a file from the filesystem
	if ( remove( filename ) == -1 ) return( FALSE );		//something has gone wrong with the deletion
	
	//it has been successfully deleted
	return( TRUE );
}

//-----------------------------------------------------------------------------------------------------------------
short fileExistsWithSize( const char* filename, long filesize ) {
	//returns TRUE if a file exists and is the same size as filesize
	FILE* handle;
	long size;
	
	handle = fopen( filename, "rb" );
	if ( handle != NULL ) {
		// the file exists
		fseek( handle, 0, SEEK_END );
		size = ftell( handle );
		fclose( handle );
		if ( size != filesize ) return( FALSE );			//file contains nothing
		
		return( TRUE );
	}
	
	//no file here
	return( FALSE );
}

//-----------------------------------------------------------------------------------------------------------------
short fileExists( char* filename ) {
	//returns TRUE if a file exists
	FILE* handle;
	long size;
	
	handle = fopen( filename, "rb" );
	if ( handle != NULL ) {
		// the file exists
		fseek( handle, 0, SEEK_END );
		size = ftell( handle );
		fclose( handle );
		if ( size == minimumFileSize ) return( FALSE );			//file contains nothing
		
		return( TRUE );
	}
	
	//no file here
	return( FALSE );
}

//-----------------------------------------------------------------------------------------------------------------
long getFilesize( char* filename ){
	//this will return the filesize or -1 if an error occured
	
	FILE* handle;
	long size;
	
	handle = fopen( filename, "rb" );
	if ( handle != NULL ){
		fseek( handle, 0, SEEK_END );
		size = ftell( handle );
		fclose( handle );
		
		return( size );
	}
	
	return( -1 );
}

//-----------------------------------------------------------------------------------------------------------------
void setupEdzUpFileSystem( char* filename ) {
	//basic test for all functionality of the local file system
	FILE* handle;
	
	handle = fopen( filename, "wb" );
	if ( handle == NULL ) {
		printf( "EDZUPFILESYSTEM:Error encountered when writing file\n" );
		minimumFileSize = 0;
	} else {
		//write an empty file to locate the size of the minium file (a file with nothing in it)
		fclose( handle );
	}
	
	handle = fopen( filename, "rb" );
	if ( handle !=NULL ) {
		fseek( handle, 0, SEEK_END );
		minimumFileSize = ftell( handle );
		fclose( handle );
		
		if ( remove( filename ) == -1 ) {						//delete the file
			printf( "ERROR: could not delete file %s\n", filename );
		}
	} else {
		printf( "EDZUPFILESYSTEM:Error encountered when reading from file\n" );
		minimumFileSize = 0;
	}
};
