/*
 *  TombstoneGlu.h
 *  FreeFalling
 *
 *  Created by EdzUp on 07/05/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

//for all OpenGL glu functions that I manage to locate or convert

#if defined( __APPLE__ )					//Apple platform uses OpenGL ES
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
#import <UIKit/UIKit.h>
#import <OpenGLES/ES1/gl.h>
#import "TombstoneiPhoneFunctions.h"
#else
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <GLUT/GLUT.h>
#import "TombstoneOSXFunctions.h"
#endif
#endif

#if defined( __LINUX__ )
#include <GL/glut.h>    // Header File For The GLUT Library 
#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#include <unistd.h>     // Header File For sleeping.
#import "TombstoneLinuxFunctions.h"		//all linux declarations for the entire engine
#endif

#if defined( __WINDOWS__ )
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library#import <windows.h>						//on windows platform import the Windows headers
#import "TombstoneWindowsFunctions.h"	//windows functions
#endif

#import <cmath>							//cmath stuff
#include <iostream>
#include <string>
#include <list>
#include <vector>

using namespace std;

string CheckErrors( void ) {
	long erra = glGetError();
	
	switch ( erra ) {
		case GL_INVALID_ENUM:return ( "ERROR:Invalid Enum" ); break;
		case GL_INVALID_VALUE:return( "ERROR:Invalid value, Out of range" ); break;
		case GL_INVALID_OPERATION:return( "ERROR:Invalid operation, Not allow in current state" );break;
		case GL_STACK_OVERFLOW: return( "ERROR:Will cause stack overflow" ); break;
		case GL_STACK_UNDERFLOW: return( "ERROR:Would cause stack underflow" ); break;
		case GL_OUT_OF_MEMORY: return( "ERROR:Out of memory for command" ); break;
//			GL_TABLE_TOO_LARGE
		default: break;
	}
	
	return( "" );
}

class gluClass {
public:
	//======================================================================================================
	void __gluMakeIdentityf(GLfloat m[16])
	{
		m[0+4*0] = 1; m[0+4*1] = 0; m[0+4*2] = 0; m[0+4*3] = 0;
		m[1+4*0] = 0; m[1+4*1] = 1; m[1+4*2] = 0; m[1+4*3] = 0;
		m[2+4*0] = 0; m[2+4*1] = 0; m[2+4*2] = 1; m[2+4*3] = 0;
		m[3+4*0] = 0; m[3+4*1] = 0; m[3+4*2] = 0; m[3+4*3] = 1;
	}
	
	//======================================================================================================
	void gluPerspective(GLfloat fovy, GLfloat aspect, GLfloat zNear, GLfloat zFar)
	{
		GLfloat m[4][4];
		GLfloat sine, cotangent, deltaZ;
		GLfloat radians = fovy / 2 * 3.14 / 180;
		
		deltaZ = zFar -zNear;
		
		sine = sin(radians);
		if ((deltaZ == 0) || (sine == 0) || (aspect == 0))
		{
			return;
		}
		cotangent = cos(radians) / sine;
		
		__gluMakeIdentityf(&m[0][0]);
		m[0][0] = cotangent / aspect;
		m[1][1] = cotangent;
		m[2][2] = -(zFar + zNear) / deltaZ;
		m[2][3] = -1;
		m[3][2] = -2 * zNear * zFar / deltaZ;
		m[3][3] = 0;
		glMultMatrixf(&m[0][0]);
	}
	
	//======================================================================================================
	void gluLookAt(GLfloat eyex, GLfloat eyey, GLfloat eyez,
				   GLfloat centerx, GLfloat centery, GLfloat centerz,
				   GLfloat upx, GLfloat upy, GLfloat upz)
	{
		GLfloat m[16];
		GLfloat x[3], y[3], z[3];
		GLfloat mag;
		
		/* Make rotation matrix */
		
		/* Z vector */
		z[0] = eyex - centerx;
		z[1] = eyey - centery;
		z[2] = eyez - centerz;
		
		mag = sqrt(z[0] * z[0] + z[1] * z[1] + z[2] * z[2]);
		if (mag) {                   /* mpichler, 19950515 */
			z[0] /= mag;
			z[1] /= mag;
			z[2] /= mag;
		}
		
		/* Y vector */
		y[0] = upx;
		y[1] = upy;
		y[2] = upz;
		
		/* X vector = Y cross Z */
		x[0] = y[1] * z[2] - y[2] * z[1];
		x[1] = -y[0] * z[2] + y[2] * z[0];
		x[2] = y[0] * z[1] - y[1] * z[0];
		
		/* Recompute Y = Z cross X */
		y[0] = z[1] * x[2] - z[2] * x[1];
		y[1] = -z[0] * x[2] + z[2] * x[0];
		y[2] = z[0] * x[1] - z[1] * x[0];
		
		/* mpichler, 19950515 */
		/* cross product gives area of parallelogram, which is < 1.0 for
		 * non-perpendicular unit-length vectors; so normalize x, y here
		 */
		
		mag = sqrt(x[0] * x[0] + x[1] * x[1] + x[2] * x[2]);
		if (mag) {
			x[0] /= mag;
			x[1] /= mag;
			x[2] /= mag;
		}
		
		mag = sqrt(y[0] * y[0] + y[1] * y[1] + y[2] * y[2]);
		if (mag) {
			y[0] /= mag;
			y[1] /= mag;
			y[2] /= mag;
		}
#define M(row,col)  m[col*4+row]
		M(0, 0) = x[0];
		M(0, 1) = x[1];
		M(0, 2) = x[2];
		M(0, 3) = 0.0;
		M(1, 0) = y[0];
		M(1, 1) = y[1];
		M(1, 2) = y[2];
		M(1, 3) = 0.0;
		M(2, 0) = z[0];
		M(2, 1) = z[1];
		M(2, 2) = z[2];
		M(2, 3) = 0.0;
		M(3, 0) = 0.0;
		M(3, 1) = 0.0;
		M(3, 2) = 0.0;
		M(3, 3) = 1.0;
#undef M
		glMultMatrixf(m);
		
		/* Translate Eye to Origin */
		glTranslatef(-eyex, -eyey, -eyez);
	}
};

//extern gluClass TombstoneGlu;

