/*
 *  TombstoneLighting.h
 *  Tombstone Demo
 *
 *  Created by EdzUp on 13/06/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

#if defined( __APPLE__ )					//Apple platform uses OpenGL ES
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
#import <UIKit/UIKit.h>
#import <OpenGLES/ES1/gl.h>
#import "TombstoneiPhoneFunctions.h"
#else
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <GLUT/GLUT.h>
#import "TombstoneOSXFunctions.h"
#endif
#endif

#if defined( __LINUX__ )
#include <GL/glut.h>    // Header File For The GLUT Library 
#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#include <unistd.h>     // Header File For sleeping.
#import "TombstoneLinuxFunctions.h"		//all linux declarations for the entire engine
#endif

#if defined( __WINDOWS__ )
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library#import <windows.h>						//on windows platform import the Windows headers
#import "TombstoneWindowsFunctions.h"	//windows functions
#endif

#include <iostream>
#include <string>
#include <list>
#import "TombstoneGlu.h"				//required for ALL MATH stuff
using namespace std;

short		LightingEnabled;

GLfloat		AmbientRed;
GLfloat		AmbientGreen;
GLfloat		AmbientBlue;
GLfloat		AmbientAlpha;

class	TLight;

class TLight {
private:
	vector< short >		enabled;
	vector< GLfloat >	X;
	vector< GLfloat >	Y;
	vector< GLfloat >	Z;
	vector< GLfloat >	pitch;
	vector< GLfloat >	yaw;
	vector< GLfloat >	roll;
	vector< GLfloat >	ambientRed;
	vector< GLfloat >	ambientGreen;
	vector< GLfloat >	ambientBlue;
	vector< GLfloat >	ambientAlpha;
	vector< GLfloat >	red;
	vector< GLfloat >	green;
	vector< GLfloat >	blue;
	
public:
	GLfloat global_ambient[ 4 ];
	
	//============================================================================
	void enableLight( int Id ) {
		//enable light in engine
		enabled[ Id ] = TRUE;
		glEnable( returnGLLight( Id ) );
	}
	
	//============================================================================
	void startupLighting( void ) {
		//enable lighting for the engine
		LightingEnabled = TRUE;
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glEnable(GL_COLOR_MATERIAL);
		
		//setup ambient colour
		AmbientRed = 0.5;
		AmbientGreen = 0.5;
		AmbientBlue = 0.5;
		AmbientAlpha = 1.0;
		global_ambient[ 0 ] = 0.5;
		global_ambient[ 1 ] = 0.5;
		global_ambient[ 2 ] = 0.5;
		global_ambient[ 3 ] = 1.0;
//		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient );
		glLightfv( GL_LIGHT0, GL_AMBIENT, global_ambient );
		
		//need to create all eight light entries
		if ( enabled.size() == 0 ) {
			//entries havent been created so create them, this is here so there isnt multiple creations of the vectors
			//if the lighting is disabled and then re-enabled.
			for ( int i=0; i<8; i++ ) {
				enabled.push_back( FALSE );
				X.push_back( 0.0 );
				Y.push_back( 0.0 );
				Z.push_back( 0.0 );
				pitch.push_back( 0.0 );
				yaw.push_back( 0.0 );
				roll.push_back( 0.0 );
				ambientRed.push_back( 0.5 );
				ambientGreen.push_back( 0.5 );
				ambientBlue.push_back( 0.5 );
				ambientAlpha.push_back( 1.0 );
				red.push_back( 1.0 );
				green.push_back( 1.0 );
				blue.push_back( 1.0 );
			}
		}
		
		//turn on first light
		enabled[ 0 ] = TRUE;
	}

	//============================================================================
	void disableLighting( void ) {
		//turn of lighting
		for ( int i = 0; i<8; i++ ) {
			if ( enabled[ i ] == TRUE ) glDisable( returnGLLight( i ) );
		}
		glDisable( GL_LIGHTING );
	}
	
	//============================================================================
	void setAmbientLight( int LightID, GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha = 2.0 ) {
		ambientRed[ LightID ] = ( 1.0 /255.0 ) *red;
		ambientGreen[ LightID ] = ( 1.0 /255.0 ) *green;
		ambientBlue[ LightID ] = ( 1.0 /255.0 ) *blue;
		if ( alpha != 2.0 ) ambientAlpha[ LightID ] = ( 1.0 /255.0 ) *alpha;			//only if alpha changed then change it
		global_ambient[ 0 ] = ambientRed[ LightID ];
		global_ambient[ 1 ] = ambientGreen[ LightID ];
		global_ambient[ 2 ] = ambientBlue[ LightID ];
		global_ambient[ 3 ] = ambientAlpha[ LightID ];
		glLightfv( returnGLLight( LightID ), GL_AMBIENT, global_ambient );
	}
	
	//============================================================================
	unsigned int returnGLLight( int LID ) {
		//return gl light relating to game system
		switch( LID ) {
			case 1 : return( GL_LIGHT1 ); break;
			case 2 : return( GL_LIGHT2 ); break;
			case 3 : return( GL_LIGHT3 ); break;
			case 4 : return( GL_LIGHT4 ); break;
			case 5 : return( GL_LIGHT5 ); break;
			case 6 : return( GL_LIGHT6 ); break;
			case 7 : return( GL_LIGHT7 ); break;
			default: return( GL_LIGHT0 ); break;
		}
		
		cout << "ERROR: there is no light with ID " << LID << " outside eight light limit!!!" << endl;
		exit( 1 );
	}
};
/*
 // Somewhere in the initialization part of your program…
 glEnable(GL_LIGHTING);
 glEnable(GL_LIGHT0);
 
 // Create light components
 Glfloat ambientLight[] = { 0.2f, 0.2f, 0.2f, 1.0f };
 Glgloat diffuseLight[] = { 0.8f, 0.8f, 0.8, 1.0f };
 Glfloat specularLight[] = { 0.5f, 0.5f, 0.5f, 1.0f };
 Glfloat position[] = { -1.5f, 1.0f, -4.0f, 1.0f };
 
 // Assign created components to GL_LIGHT0
 glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
 glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
 glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
 glLightfv(GL_LIGHT0, GL_POSITION, position);
 
 // ...Somewhere during initialization...
 
 // enable color tracking
 glEnable(GL_COLOR_MATERIAL);
 // set material properties which will be assigned by glColor
 glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
 
 
 // ...Somewhere in the main rendering loop of your program...
 // Draw a polygon with material properties set by glColor
 
 glColor3f(0.0f, 0.0f, 1.0f); // blue reflective properties
 glBegin(GL_TRIANGLES);
 glVertex3f(-1.0f, 0.0f, 0.0f);
 glVertex3f(0.0f, -1.0f, 0.0f);
 glVertex3f(1.0f, 0.0f, 0.0f);
 glEnd();
 
 SHININESS:
 float specReflection[] = { 0.8f, 0.8f, 0.8f, 1.0f };
 glMaterialfv(GL_FRONT, GL_SPECULAR, specReflection);
 
 glMateriali(GL_FRONT, GL_SHININESS, 96);
*/
