/*
 *  TombstoneExternalVariables.h
 *  Tombstone Demo
 *
 *  Created by EdzUp on 25/06/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */

#import "TombstoneStringFunctions.h"
#import "TTexture.h"
#import "TombstoneRandomSystem.h"

//These are the external variables that will be used in coding :)

long gNumSprites;

#define FLIP_SPRITE_VERTICAL		1
#define FLIP_SPRITE_HORIZONTAL		2
#define FLIP_SPRITE_BOTH			3

//======================================================================================================
void setSpriteRotation( GLfloat angle ) {
	imageDrawAngle = angle;
}

class TSpriteClass {
public:
	long		ID;
	TTexture	name;
	short		textureUnique;
	GLfloat		verts[12];
	GLfloat		tverts[8];
	
	long		maximumFrames;
	short		animated;				//true if this is animated
	GLfloat		animatedWidth;
	GLfloat		animatedHeight;
/*
	//======================================================================================================
	void updateTexture( GLuint newTexture ) {
		//required for adjustment system
		name = newTexture;
	}
*/
	
	//======================================================================================================	
	void FreeSprite( void ) {
		if ( textureUnique == TRUE ) {
			name.FreeTexture();
		}
		for ( int fs = 0; fs <12; fs++ ) {
			verts[ fs ] = 0.0;
		}
		for ( int fs = 0; fs <8; fs++ ) {
			tverts[ fs ] = 0.0;
		}
	}
	
	//======================================================================================================
	void updateSpriteData( const TDataBank imageData ) {
		//this will replace the OpenGL texture with one we have
		// Bind the number of textures we need, in this case one.
		
		//Sprites, Models, Surfaces will need updating too to use the new entry.
		name.updateTextureData( imageData );
	}
	
	//======================================================================================================
	void GrabSpriteSet(TTexture imageFrom, float startX, float startY, int across, int down, float width, float height, int count, TSpriteClass spriteArray[] ) {
		int sID = 0;
		int x,y;
		float	cX, cY;
		
		cX = 0;
		cY = 0;
		for(y = 0; y < down && count > 0 ;y++)
		{
			for(x = 0; x < across && count > 0;x++)
			{
				spriteArray[ sID ].SetSpriteUV( imageFrom, (int)cX, (int)cY, (int)width, (int)height, TRUE );
				cX+=width;
				sID++;
				count--;
			}
			cX = 0;
			cY+=height;
		}
	}	
	
	//======================================================================================================
	void spriteFlipHorizontal( void ) {
		//flip the sprite horizontal
		GLfloat tempU, tempV;
		
		//first UV
		tempU = tverts[ 0 ];
		tempV = tverts[ 1 ];
		tverts[ 0 ] = tverts[ 2 ];
		tverts[ 1 ] = tverts[ 3 ];
		tverts[ 2 ] = tempU;
		tverts[ 3 ] = tempV;
		
		//second set
		tempU = tverts[ 4 ];
		tempV = tverts[ 5 ];
		tverts[ 4 ] = tverts[ 6 ];
		tverts[ 5 ] = tverts[ 7 ];
		tverts[ 6 ] = tempU;
		tverts[ 7 ] = tempV;
	}
	
	//======================================================================================================
	void spriteFlipVertical( void ) {
		//flip sprite vertical
		GLfloat tempU, tempV;
		
		//first set
		tempU = tverts[ 0 ];
		tempV = tverts[ 1 ];
		tverts[ 0 ] = tverts[ 4 ];
		tverts[ 1 ] = tverts[ 5 ];
		tverts[ 4 ] = tempU;
		tverts[ 5 ] = tempV;
		
		//second set
		tempU = tverts[ 2 ];
		tempV = tverts[ 3 ];
		tverts[ 2 ] = tverts[ 6 ];
		tverts[ 3 ] = tverts[ 7 ];
		tverts[ 6 ] = tempU;
		tverts[ 7 ] = tempV;
	}
		
	//======================================================================================================
	void spriteFlipX( void ) {
		//flip the sprite horizontal
		GLfloat tempU, tempV;
		
		//first UV
		tempU = tverts[ 0 ];
		tempV = tverts[ 1 ];
		tverts[ 0 ] = tverts[ 2 ];
		tverts[ 1 ] = tverts[ 3 ];
		tverts[ 2 ] = tempU;
		tverts[ 3 ] = tempV;
		
		//second set
		tempU = tverts[ 4 ];
		tempV = tverts[ 5 ];
		tverts[ 4 ] = tverts[ 6 ];
		tverts[ 5 ] = tverts[ 7 ];
		tverts[ 6 ] = tempU;
		tverts[ 7 ] = tempV;
	}

	//======================================================================================================
	void spriteFlipY( void ) {
		//flip sprite vertical
		GLfloat tempU, tempV;
		
		//first set
		tempU = tverts[ 0 ];
		tempV = tverts[ 1 ];
		tverts[ 0 ] = tverts[ 4 ];
		tverts[ 1 ] = tverts[ 5 ];
		tverts[ 4 ] = tempU;
		tverts[ 5 ] = tempV;
		
		//second set
		tempU = tverts[ 2 ];
		tempV = tverts[ 3 ];
		tverts[ 2 ] = tverts[ 6 ];
		tverts[ 3 ] = tverts[ 7 ];
		tverts[ 6 ] = tempU;
		tverts[ 7 ] = tempV;
	}
	
	//======================================================================================================
	void spriteFlip( const long options ) {
		switch( options ) {
			case FLIP_SPRITE_HORIZONTAL: spriteFlipX(); break;
			case FLIP_SPRITE_VERTICAL  : spriteFlipY(); break;
			default: spriteFlipX();
				spriteFlipY(); break;
		}
	}
	
	//======================================================================================================
	bool hasEnding (const std::string& fullString, string const &ending )
	{
		if (fullString.length() >= ending.length()) {
			return (0 == fullString.compare (fullString.length() - ending.length(), ending.length(), ending));
		} else {
			return false;
		}
	}
	
	//======================================================================================================
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
	CGImageRef CreateNamedImage(CFStringRef imageName, CFStringRef type) {
		//called first to create the basics
		CGImageRef img = NULL;
		
		CFURLRef url = CFBundleCopyResourceURL(CFBundleGetMainBundle(),imageName,type,NULL);
		if(url != NULL)
		{
			CGDataProviderRef imgSrc = CGDataProviderCreateWithURL(url);
			if(imgSrc != NULL)
			{
				img = CGImageCreateWithPNGDataProvider(imgSrc,NULL,false,kCGRenderingIntentDefault);
				CFRelease(imgSrc);
			}
			else
			{
				printf("imgSrc = null for %s.%s\n",CFStringGetCStringPtr(imageName,kCFStringEncodingASCII),CFStringGetCStringPtr(type,kCFStringEncodingASCII));
			}
			CFRelease(url);
		}
		else
		{
			printf("url = null for %s.%s\n",CFStringGetCStringPtr(imageName,kCFStringEncodingASCII),CFStringGetCStringPtr(type,kCFStringEncodingASCII));
		}
		return img;
	}
#endif
	
#if TARGET_OS_SIMULATOR || TARGET_OS_IPHONE
	//======================================================================================================
	void LoadSprite( const std::string& filename ) {
		//filename, type are just strings ie 'Jumper', 'png'
		//sprname is file variable it is to be loaded into :)
		cout << "Loading texture:" << filename << "...";
		name.loadTexture( filename );
		CreateSprite( name );
		if ( name.Tname == 0x0 ) {
			cout << "ERROR LOADING" << endl;
			exit( 1 );
		} else {
			cout << "COMPLETED" << endl;
			return;
		}
		
		string	extension = "";
		long	dotLocation;
		
		//
		cout << "LOADING SPRITE:" << filename << endl;
		//get location of . in filename to load
		dotLocation = InStr( filename, ".", filename.size()-5 );
		if ( dotLocation != -1 ) {
			//get the extension
			extension = Right( filename, filename.size() -dotLocation );
		}
		
		if ( hasEnding( filename, ".png" ) == TRUE ) {
			NSUInteger				width,								//Width of an image
			height,														//height of an image
			i;
			CGContextRef			context = nil;						//its context
			void*					data = nil;;						//the image data
			CGColorSpaceRef			colorSpace;							//its colour space reference
			void*					tempData;
			unsigned int*			inPixel32;
			unsigned short*			outPixel16;
			BOOL					hasAlpha;
			CGImageAlphaInfo		info;								//image alpha information
			CGAffineTransform		transform;
			CGSize					imageSize;
			Texture2DPixelFormat    pixelFormat;
			CGImageRef				image;
			UIImageOrientation		orientation;
			BOOL					sizeToFit = NO;
			
			string					ext = getExtension( filename );
			string					nameOfFile ( filename.begin(), filename.end() -ext.size() );
			CFStringRef				Name;
			CFStringRef				type;			
			
			if ( hasEnding( ext, ".png" ) == TRUE ) {
				//image is a png now convert the strings to CFStringRef for 'Apple'
				Name = CFStringCreateWithCString( kCFAllocatorDefault, 
												 nameOfFile.c_str(), 
												 kCFStringEncodingASCII);
				type = CFSTR( "png" );
			}
		
			image = CreateNamedImage( Name, type );					//create and LOAD the image this is then returned
			orientation = UIImageOrientationUp;							//adjust orientation
			if(image == NULL) 
			{
				cout << "LoadSprite: Error loading sprite " << filename << endl;
				exit(1);
			}
		
			info = CGImageGetAlphaInfo(image);
			hasAlpha = ((info == kCGImageAlphaPremultipliedLast) || (info == kCGImageAlphaPremultipliedFirst) || (info == kCGImageAlphaLast) || (info == kCGImageAlphaFirst) ? YES : NO);
			if(CGImageGetColorSpace(image)) {
				if(hasAlpha)
					pixelFormat = kTexture2DPixelFormat_RGBA8888;
				else
					pixelFormat = kTexture2DPixelFormat_RGB565;
			} 
			else  //NOTE: No colorspace means a mask image
			{
				pixelFormat = kTexture2DPixelFormat_A8;
				imageSize.width *= 0.5;
			}
		
			imageSize = CGSizeMake(CGImageGetWidth(image), CGImageGetHeight(image));
			transform = CGAffineTransformIdentity;
		
			width = imageSize.width;
			// round up to nearest power of 2
			if((width != 1) && (width & (width - 1))) {
				i = 1;
				while((sizeToFit ? 2 * i : i) < width)
					i *= 2;
				width = i;
			}
			height = imageSize.height;
			// round up to nearest power of 2
			if((height != 1) && (height & (height - 1))) {
				i = 1;
				while((sizeToFit ? 2 * i : i) < height)
					i *= 2;
				height = i;
			}
		
			// check texture size is within maximum texture size (1024x1024), although if you're loading in textures this big you're going to have memory issues anyway
			while((width > kMaxTextureSize) || (height > kMaxTextureSize)) {
				width /= 2;
				height /= 2;
				transform = CGAffineTransformScale(transform, 0.5, 0.5);
				imageSize.width *= 0.5;
				imageSize.height *= 0.5;
			}
		
			//setup the pixel format
			switch(pixelFormat) {		
				case kTexture2DPixelFormat_RGBA8888:
					colorSpace = CGColorSpaceCreateDeviceRGB();
					data = calloc(height * width * 4,1);
					context = CGBitmapContextCreate(data, width, height, 8, 4 * width, colorSpace, kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big);
					CGColorSpaceRelease(colorSpace);
					break;
				case kTexture2DPixelFormat_RGB565:
					colorSpace = CGColorSpaceCreateDeviceRGB();
					data = calloc(height * width * 4,1);
					context = CGBitmapContextCreate(data, width, height, 8, 4 * width, colorSpace, kCGImageAlphaNoneSkipLast | kCGBitmapByteOrder32Big);
					CGColorSpaceRelease(colorSpace);
					break;
				
				case kTexture2DPixelFormat_A8:
					data = calloc(height * width,1);
					context = CGBitmapContextCreate(data, width, height, 8, width, NULL, kCGImageAlphaOnly);
					break;				
				default:
					printf( "Invalid texture format\n" );
					exit( 1 );
			}

			CGContextClearRect(context, CGRectMake(0, 0, width, height));
			CGContextTranslateCTM(context, 0, height - imageSize.height);
		
			if(!CGAffineTransformIsIdentity(transform))
				CGContextConcatCTM(context, transform);
			CGContextDrawImage(context, CGRectMake(0, 0, CGImageGetWidth(image), CGImageGetHeight(image)), image);
				
			//Convert "RRRRRRRRRGGGGGGGGBBBBBBBBAAAAAAAA" to "RRRRRGGGGGGBBBBB"
			if(pixelFormat == kTexture2DPixelFormat_RGB565) {
				tempData = calloc(height * width * 2,1);
				inPixel32 = (unsigned int*)data;
				outPixel16 = (unsigned short*)tempData;
				for(i = 0; i < width * height; ++i, ++inPixel32)
					*outPixel16++ = ((((*inPixel32 >> 0) & 0xFF) >> 3) << 11) | ((((*inPixel32 >> 8) & 0xFF) >> 2) << 5) | ((((*inPixel32 >> 16) & 0xFF) >> 3) << 0);
				free(data);
				data = tempData;
			
			}
		
			//need to locate a free entry for the texture system to stop any problems
		
			//		printf("texture loaded, w/h = %d,%d to  %li\n",width,height,gNumTextures);
		
			//create the entry
			InitLSTexture(data,pixelFormat,width,height,imageSize,NULL);
		
			CGContextRelease(context);
			free(data);
			CGImageRelease(image);
			
			animated = FALSE;
			maximumFrames = 0;
			textureUnique = TRUE;
		}
		
		if ( name.Tname == 0x0 ) {
			cout << "Error: Could not load texture " << filename << endl;
		}
	}
#endif
	
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
	//======================================================================================================
	//textureClass 
	void InitLSTexture(const void* data, Texture2DPixelFormat pixelFormat, NSUInteger width, NSUInteger height, CGSize size,int sprname){		
		glGenTextures(1, &name.Tname );											//generate a texture object in OpenGL
		string err = CheckErrors();
		if ( err != "" ) {
			cout << "BOUND Count:" << texturesBound << endl; 
			cout << err << endl;
			exit( 1 );
		}
		
		glBindTexture(GL_TEXTURE_2D, name.Tname );									//bind the object (in 2d)
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);	//setup the texture parameters
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		
		texturesBound++;
		
		//THIS CREATES THE TEXTURE FOR USAGE
		switch(pixelFormat) 
		{
			case kTexture2DPixelFormat_RGBA8888:
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
				break;
			case kTexture2DPixelFormat_RGB565:
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, data);
				break;
			case kTexture2DPixelFormat_A8:
				glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, width, height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, data);
				break;
			default:
				printf( "Invalid texture format\n" );
				exit( 1 );
				
		}
		
		//setup its details
		ID = gNumSprites;
//		ret->size = size;
//		ret->width = width;
//		ret->height = height;
//		ret->format = pixelFormat;
		float maxS = size.width / (float)width;
		float maxT = size.height / (float)height;
		
		//create the sprite
		CreateLSSprite( gNumSprites, width *maxS, height *maxT, 0, 0, maxS, maxT, name.Tname, sprname );
	}
#endif
	
	//======================================================================================================
	void CreateLSSprite(int a,float width,float height,float u1,float v1,float u2,float v2,GLuint tex, int sprname ) {
		int b;
		GLfloat		coordinates[8] = { u1,v2,
			u2,	v2,
			u1,		v1,
			u2,	v1 };
		GLfloat		vertices[12] = {	-width / 2,	-height / 2 ,	0.0,
			width / 2 ,	-height / 2,	0.0,
			-width / 2 ,	height / 2 ,	0.0,
			width / 2 ,	height / 2 ,	0.0 };
				
		//now we setup the verts and coords we need to create the sprite
		ID = a;
		name.Tname = tex;
		
		for(b = 0; b < 8;b++)
		{
			tverts[ b ] = coordinates[ b ];
		}
		for(b = 0; b < 12;b++)
		{
			verts[ b ] = vertices[ b ];
		}
	}
	
	//======================================================================================================
	void CreateSprite( const TTexture entry ) {
		//create the sprite
		GLfloat		u1, v1, u2, v2;
		u1 = 0.0;
		v1 = 0.0;
		u2 = entry.TmaxS;
		v2 = entry.TmaxT;
		
		GLfloat		width = entry.Twidth *entry.TmaxS;
		GLfloat		height = entry.Theight *entry.TmaxT;
		GLfloat		coordinates[8] = { u1,v2,
			u2,	v2,
			u1,		v1,
			u2,	v1 };
		GLfloat		vertices[12] = {	-width / 2,	-height / 2 ,	0.0,
			width / 2 ,	-height / 2,	0.0,
			-width / 2 ,	height / 2 ,	0.0,
			width / 2 ,	height / 2 ,	0.0 };
		
		//now we setup the verts and coords we need to create the sprite
		ID = gNumSprites;
		name = entry;
		for(int b = 0; b < 8;b++)
		{
			tverts[ b ] = coordinates[ b ];
		}
		for(int b = 0; b < 12;b++)
		{
			verts[ b ] = vertices[ b ];
		}
		gNumSprites++;
		textureUnique = FALSE;			//created from a texture
	}

	//======================================================================================================
	void PasteSprite( GLfloat x, GLfloat y, GLfloat z = 0.0, GLfloat pitch = 0.0, GLfloat yaw = 0.0, GLfloat roll = 1.0 ) {
		if ( name.Tname == 0x0 ) {
			cout << "Paste Sprite contains no data Texture filename " << name.Tname << endl;
//			exit( 1 );
			return;
		}
		if( name.Tname == 0x0 ) {
			//no sprite present
//			cout << "Empty sprite handle for texture filename " << name.Tname << endl;
//			exit( 1 );
			return;
		}
		else
		{
			glPushMatrix();
			glTranslatef( x +TombstoneOffsetX, y +TombstoneOffsetY, z +TombstoneOffsetZ );
			if ( pitch == 0.0 && yaw == 0.0 && roll == 1.0 ) {
				glRotatef( imageDrawAngle, 0.0, 0.0, 1.0 );						//rotate to required angle (angle, X, Y, Z )
			} else {
				glRotatef( pitch, 1.0, 0.0, 0.0 );
				glRotatef( yaw, 0.0, 1.0, 0.0 );
				glRotatef( roll, 0.0, 0.0, 1.0 );
			}
			glColor4f( GLRed, GLGreen, GLBlue, GLAlpha );					//colour to required rgba value
			glBindTexture( GL_TEXTURE_2D, name.Tname );					//bind its texture
			glVertexPointer( 3, GL_FLOAT, 0, verts );				//setup vertices
			glTexCoordPointer( 2, GL_FLOAT,0, tverts );				//create triangles
			glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );						//create quad
			glPopMatrix();
		}
	}
	
	//======================================================================================================
	void PasteScaledSprite( GLfloat x, GLfloat y, GLfloat sx, GLfloat sy ) {
		glPushMatrix();
		glTranslatef( x +TombstoneOffsetX, y +TombstoneOffsetY, 0+TombstoneOffsetZ );
		glScalef( sx, sy, 1 );
		if( name.Tname == 0 )
		{
			printf( "image %li is not a valid image, it contains no vertex data", ID );
		}
		else
		{
			glRotatef( imageDrawAngle, 0.0, 0.0, 1.0 );
			glColor4f( GLRed, GLGreen, GLBlue, GLAlpha );
			glBindTexture(GL_TEXTURE_2D,name.Tname);
			glVertexPointer(3,GL_FLOAT,0,verts);
			glTexCoordPointer(2,GL_FLOAT,0,tverts);
			glDrawArrays(GL_TRIANGLE_STRIP,0,4);
		}
		glPopMatrix();
	}

	//======================================================================================================
	void HorizontalWave( long xW, long yW, float startAngle = 0.0, float increment = 5.0, float radius = 5.0, long segmentSize = 1, short fadeMode = FADE_NOFADE ) {
		float	HWAngle;
		int		HWPos;
		float	fadeAlpha = 1.0;
		float	fadeAddition = 0.0;
		
		if ( name.Tname == 0x0 ) {
			cout << "Sprite Effect: no texture specified" << endl;
			return;
		}
		
		yW -= name.Theight /2;
		
		switch ( fadeMode ) {
			case FADE_FROMTOP:
				fadeAlpha = 0.0;
				fadeAddition = ( 1.0 /name.Theight ) *2;
				break;
			case FADE_TOMIDDLE:
				fadeAlpha = 0.0;
				fadeAddition = ( 1.0 /name.Theight ) *4;
				break;
			case FADE_FROMBOTTOM:
				fadeAlpha = 1.0;
				fadeAddition = 0 -( ( 1.0 /name.Theight ) *2 );
				break;
			case FADE_FROMMIDDLE:
				fadeAlpha = 1.0;
				fadeAddition = 0 -( ( 1.0 /name.Theight ) *4 );
				break;
		}

		HWAngle = startAngle;
		HWPos = yW+name.Theight;//yW;
		for ( long pos = 0; pos<(long)name.Theight; pos += ( segmentSize +0 ) ) {
			if ( fadeMode == FADE_RANDOM ) fadeAlpha = TombstoneRandom.randFloat( 0.0, 1.0 );
			GLAlpha = fadeAlpha;
			fadeAlpha += fadeAddition;
			switch ( fadeMode ) {
				case FADE_TOMIDDLE:
					if ( fadeAlpha >= 1.0 ) {
						fadeAddition = 0 -fadeAddition;
					}
					break;
				case FADE_FROMMIDDLE:
					if ( fadeAlpha <= 0.0 ) {
						fadeAddition = 0 -fadeAddition;
					}
					break;
			}
			SetSpriteUV( name, 0, pos, name.Twidth, segmentSize +1 );
			PasteSprite( xW+( sin( HWAngle *PI/180 ) *radius ), HWPos );
			HWAngle += increment;
			HWPos -= ( segmentSize +0 );
		}
		GLAlpha = 1.0;
	}
	
	//======================================================================================================
	void VerticalWave( long xW, long yW, float startAngle = 0.0, float increment = 5.0, float radius = 5.0, long segmentSize = 1, short fadeMode = FADE_NOFADE ) {
	}

	//======================================================================================================
	void ScaleSpriteTexture(float u_s, float v_s, int alignment = alignTopLeft ){
		GLfloat u1, u2, v1, v2;
		GLfloat coordinates[8];
				
		switch ( alignment ) {
			case alignTopCenter:
				u1 = 0.5 -( ( 1.0 /u_s ) *.5 );
				v1 = 0.0;
				u2 = 0.5 +( ( 1.0 /u_s ) *.5 );
				v2 = 1.0 /v_s; break;
			case alignTopRight:
				u1 = 1.0 - ( 1.0 /u_s );
				v1 = 0.0;
				u2 = 1.0;
				v2 = 1.0 /v_s; break;
			case alignLeft:
				u1 = 0.0;
				v1 = 0.5 -( ( 1.0 /v_s ) *.5 );
				u2 = 1.0 /u_s;
				v2 = 0.5 +( ( 1.0 /v_s ) *.5 ); break;
			case alignCenter:
				u1 = 0.5 -( ( 1.0 /u_s ) *.5 );
				v1 = 0.5 -( ( 1.0 /v_s ) *.5 );
				u2 = 0.5 +( ( 1.0 /u_s ) *.5 );
				v2 = 0.5 +( ( 1.0 /v_s ) *.5 ); break;
			case alignRight:
				u1 = 1.0 - ( 1.0 /u_s );
				v1 = 0.5 -( ( 1.0 /v_s ) *.5 );
				u2 = 1.0;
				v2 = 0.5 +( ( 1.0 /v_s ) *.5 ); break;
			case alignBottomLeft:
				u1 = 0.0;
				v1 = 1.0-( 1.0 / v_s );
				u2 = 1.0 /u_s;
				v2 = 1.0; break;				
			case alignBottomCenter:
				u1 = 0.5 -( ( 1.0 /u_s ) *.5 );
				v1 = 1.0-( 1.0 / v_s );
				u2 = 0.5 +( ( 1.0 /u_s ) *.5 );
				v2 = 1.0; break;
			case alignBottomRight:
				u1 = 1.0 - ( 1.0 /u_s );
				v1 = 1.0 - ( 1.0 /v_s );
				u2 = 1.0;
				v2 = 1.0; break;				
			default:				//align Top Left
				u1 = 0.0;
				v1 = 0.0;
				u2 = 1.0 /u_s;
				v2 = 1.0 /v_s; break;
		}
		
		coordinates[0] = u1;
		coordinates[1] = v2;
		coordinates[2] = u2;
		coordinates[3] = v2;
		coordinates[4] = u1;
		coordinates[5] = v1;
		coordinates[6] = u2;
		coordinates[7] = v1;
		
		for(int b = 0; b < 8;b++)
		{
			tverts[ b ] = coordinates[ b ];
		}
	}
	
	//======================================================================================================
	void PositionSpriteTexture(float u_p,float v_p){
		
		//		u_pos=-u_p;
		//		v_pos=-v_p;
		
	}
	
	//======================================================================================================
	void SetSpriteUV( const TTexture imageFrom, int x, int y, int width, int height, short creation = FALSE ) {
		//this will reset a sprites UV coordinates to something within a texture image, useful for users own systems
		if ( creation == TRUE ) {
			CreateSprite( imageFrom );
		}
		
		if ( width == 0 || height == 0 ) {
			cout << "Set Sprite UV Error: Cannot have width or height of 0" << endl;
			return;
		}
		
		GLfloat u1, u2, v1, v2;
		GLfloat coordinates[8];
		GLfloat		vertices[12] = {	-width / 2,	-height / 2 ,	0.0,
			width / 2 ,	-height / 2,	0.0,
			-width / 2 ,	height / 2 ,	0.0,
			width / 2 ,	height / 2 ,	0.0 };
				
		GLfloat SSUVPercentX = 1.0 /imageFrom.Twidth;
		GLfloat SSUVPercentY = 1.0 /imageFrom.Theight;
		
		name = imageFrom;
		
		//work out the UV Coordinates
		u1 = ( ( x *SSUVPercentX ) );
		v1 = ( ( y *SSUVPercentY ) );
		u2 = ( ( ( x +width ) *SSUVPercentX ) );		//must be x+width as we want to get the proper width etc
		v2 = ( ( ( y +height ) *SSUVPercentY ) );
		
		coordinates[0] = u1;
		coordinates[1] = v2;
		coordinates[2] = u2;
		coordinates[3] = v2;
		coordinates[4] = u1;
		coordinates[5] = v1;
		coordinates[6] = u2;
		coordinates[7] = v1;
		
		for(int b = 0; b < 8;b++)
		{
			tverts[ b ] = coordinates[ b ];
		}
		for(int b = 0; b < 12;b++)
		{
			verts[ b ] = vertices[ b ];
		}
	}
};

typedef TSpriteClass TSprite;
