/*
 *  TTexture.h
 *  Tombstone Demo
 *
 *  Created by EdzUp on 25/06/2010.
 *  Copyright 2010 Graveyard Dogs. All rights reserved.
 *
 */
#define FULLSCREEN_NOCHANGE			0			//if nothing is to be done
#define FULLSCREEN_RANDOMRED		1			//Randomize the red quotient
#define	FULLSCREEN_RANDOMGREEN		2			//Randomize the green quotient
#define FULLSCREEN_RANDOMBLUE		3			//Randomize the blue quotient
#define	FULLSCREEN_RANDOMALPHA		4			//Randomize the alpha quotient

#if defined( __APPLE__ )					//Apple platform uses OpenGL ES
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
#import <UIKit/UIKit.h>
#import <OpenGLES/ES1/gl.h>
#import "TombstoneiPhoneFunctions.h"
#else
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <GLUT/GLUT.h>
#import "TombstoneOSXFunctions.h"
#endif
#endif

#if defined( __LINUX__ )
#include <GL/glut.h>    // Header File For The GLUT Library 
#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#include <GL/glut.h>
#include <unistd.h>     // Header File For sleeping.
#import "TombstoneLinuxFunctions.h"		//all linux declarations for the entire engine
#endif

#if defined( __WINDOWS__ )
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glext.h>
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library
#include <gl\glut.h>
#import <windows.h>						//on windows platform import the Windows headers
#import "TombstoneWindowsFunctions.h"	//windows functions
#endif

#include <iostream>
#include <cstdio>
#include <string>
#include <list>
#import "TColour.h"
#import "TombstoneGlu.h"				//required for ALL MATH stuff
#import "TombstoneStringFunctions.h"
#import "TombstoneRandomSystem.h"
#include <math.h>

using namespace std;

#define TEXTURE_LOAD_ERROR 0

long gNumTextures;						//for Texture ID's

class TTextureClass {
private:
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
	UIImage* glToUIImage( long width, long height ) {
		NSInteger myDataLength = width * height * 4;
		
		// allocate array and read pixels into it.
		GLubyte *buffer = (GLubyte *) malloc(myDataLength);
		glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
		
		// gl renders "upside down" so swap top to bottom into new array.
		// there's gotta be a better way, but this works.
		GLubyte *buffer2 = new GLubyte[ myDataLength ];
		for(int y = 0; y <height; y++)
		{
			for(int x = 0; x <width * 4; x++)
			{
				buffer2[( (height -1) - y) * width * 4 + x] = buffer[y * 4 * width + x];
			}
		}
		
		// make data provider with data.
		CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, buffer2, myDataLength, NULL);
		
		// prep the ingredients
		int bitsPerComponent = 8;
		int bitsPerPixel = 32;
		int bytesPerRow = 4 * width;
		CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateDeviceRGB();
		CGBitmapInfo bitmapInfo = kCGBitmapByteOrderDefault;
		CGColorRenderingIntent renderingIntent = kCGRenderingIntentDefault;
		
		// make the cgimage
		CGImageRef imageRef = CGImageCreate(width, height, bitsPerComponent, bitsPerPixel, bytesPerRow, colorSpaceRef, bitmapInfo, provider, NULL, NO, renderingIntent);
		
		// then make the uiimage from that
		UIImage *myImage = [UIImage imageWithCGImage:imageRef];
		
		delete[] buffer2;
		
		return myImage;
	}
#endif
	
public:
	gluClass	TombstoneGlu;
	long		TID;
	long		rawSize;
	GLuint		Tname;
	CGSize		Tsize;
	NSUInteger	Twidth, Theight;
	Texture2DPixelFormat Tformat;
	GLfloat		TmaxS, TmaxT;
	GLuint		FrameBufferObject;
	GLuint		DepthRenderBuffer;

	/*
	 
	 */

	//======================================================================================================
	void createTexture( long width, long height )							// Create An Empty Texture
	{
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
		void* data = nil;
		CGSize size;
		
		data = calloc( width * height *4, 1 );
		size.width = width;
		size.height = height;
		Twidth = width;
		Theight = height;
				
		InitTexture( data, kTexture2DPixelFormat_RGBA8888, width, height, size );
		
		free( data );
#else
#endif
	}

	//======================================================================================================
	void updateTextureData( const TDataBank imageData ) {
		//this will replace the OpenGL texture with one we have
		// Bind the number of textures we need, in this case one.
		
		//Sprites, Models, Surfaces will need updating too to use the new entry.
		
		glDeleteTextures( 1, &Tname );				//Delete the previous texture
		
		glGenTextures(1, &Tname );					//Generate a new texture entry
		glBindTexture(GL_TEXTURE_2D, Tname );		//Bind it to OpenGL
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	//setup the texture parameters
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, imageData.width, imageData.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, &imageData.data );//create from imageData
	}
	
#if !TARGET_OS_IPHONE && !TARGET_OS_SIMULATOR
	// load a 256x256 RGB .RAW file as a texture
	void loadTextureRAW( const std::string& filename, short byteCount = 3, long RAWwidth=256, long RAWheight=256, int wrap=FALSE )
	{
		int width, height;
		unsigned short * data;
		FILE * file;
		
		// open texture data
		file = fopen( filename.c_str(), "rb" );
		if ( file == NULL ) {
			//Error
			cout << "RAW Loader error: Could not read from file" << endl;
			exit( 1 );
		}
		
		// allocate buffer
		width = RAWwidth;
		height = RAWheight;
		data = (unsigned short*) malloc( width * height * byteCount );
		
		// read texture data
		fread( data, width * height * byteCount, 1, file );
		fclose( file );
		
		// allocate a texture name
		glGenTextures( 1, &Tname );
		
		// select our current texture
		glBindTexture( GL_TEXTURE_2D, Tname );
		
		// select modulate to mix texture with color for shading
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
		
		// when texture area is small, bilinear filter the closest mipmap
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
						GL_LINEAR_MIPMAP_NEAREST );
		// when texture area is large, bilinear filter the first mipmap
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		
		// if wrap is true, the texture wraps over at the edges (repeat)
		//       ... false, the texture ends at the edges (clamp)
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
						wrap ? GL_REPEAT : GL_CLAMP );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
						wrap ? GL_REPEAT : GL_CLAMP );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		
//		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, &data );

		// build our texture mipmaps
		if ( byteCount == 3 ) {
			gluBuild2DMipmaps( GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data );
		} else {
			gluBuild2DMipmaps( GL_TEXTURE_2D, 4, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data );
		}
		cout << "RAW Loading complete..." << endl;
		// free buffer
		free( data );
	}
#endif

	//======================================================================================================
	void loadFile(std::vector<unsigned char>& buffer, const std::string& filename) //designed for loading files from hard disk in an std::vector
	{
		FILE* handle;
		
		//make sure the file exists
		handle = fopen( filename.c_str(), "rb" );
		if ( handle == NULL ) {
			cout << "PNG loadFile Error: no file located" << endl;
			exit( 1 );
		} else {
			fclose( handle );
		}
		
		std::ifstream file(filename.c_str(), std::ios::in|std::ios::binary|std::ios::ate);
		
		//get filesize
		std::streamsize size = 0;
		if(file.seekg(0, std::ios::end).good()) size = file.tellg();
		if(file.seekg(0, std::ios::beg).good()) size -= file.tellg();
		
		//read contents of the file into the vector
		if(size > 0)
		{
			buffer.resize((size_t)size);
			file.read((char*)(&buffer[0]), size);
		} else {
			buffer.clear();
			cout << "PNG loadFile Error: no file size" << endl;
			exit( 1 );
		}
	}
	
	/*
	 decodePNG: The picoPNG function, decodes a PNG file buffer in memory, into a raw pixel buffer.
	 out_image: output parameter, this will contain the raw pixels after decoding.
	 By default the output is 32-bit RGBA color.
	 The std::vector is automatically resized to the correct size.
	 image_width: output_parameter, this will contain the width of the image in pixels.
	 image_height: output_parameter, this will contain the height of the image in pixels.
	 in_png: pointer to the buffer of the PNG file in memory. To get it from a file on
	 disk, load it and store it in a memory buffer yourself first.
	 in_size: size of the input PNG file in bytes.
	 convert_to_rgba32: optional parameter, true by default.
	 Set to true to get the output in RGBA 32-bit (8 bit per channel) color format
	 no matter what color type the original PNG image had. This gives predictable,
	 useable data from any random input PNG.
	 Set to false to do no color conversion at all. The result then has the same data
	 type as the PNG image, which can range from 1 bit to 64 bits per pixel.
	 Information about the color type or palette colors are not provided. You need
	 to know this information yourself to be able to use the data so this only
	 works for trusted PNG files. Use LodePNG instead of picoPNG if you need this information.
	 return: 0 if success, not 0 if some error occured.
	 */
	int decodePNG(std::vector<unsigned char>& out_image, unsigned long& image_width, unsigned long& image_height, const unsigned char* in_png, size_t in_size, bool convert_to_rgba32 = true)
	{
		// picoPNG version 20101224
		// Copyright (c) 2005-2010 Lode Vandevenne
		//
		// This software is provided 'as-is', without any express or implied
		// warranty. In no event will the authors be held liable for any damages
		// arising from the use of this software.
		//
		// Permission is granted to anyone to use this software for any purpose,
		// including commercial applications, and to alter it and redistribute it
		// freely, subject to the following restrictions:
		//
		//     1. The origin of this software must not be misrepresented; you must not
		//     claim that you wrote the original software. If you use this software
		//     in a product, an acknowledgment in the product documentation would be
		//     appreciated but is not required.
		//     2. Altered source versions must be plainly marked as such, and must not be
		//     misrepresented as being the original software.
		//     3. This notice may not be removed or altered from any source distribution.
		
		// picoPNG is a PNG decoder in one C++ function of around 500 lines. Use picoPNG for
		// programs that need only 1 .cpp file. Since it's a single function, it's very limited,
		// it can convert a PNG to raw pixel data either converted to 32-bit RGBA color or
		// with no color conversion at all. For anything more complex, another tiny library
		// is available: LodePNG (lodepng.c(pp)), which is a single source and header file.
		// Apologies for the compact code style, it's to make this tiny.
		
		static const unsigned long LENBASE[29] =  {3,4,5,6,7,8,9,10,11,13,15,17,19,23,27,31,35,43,51,59,67,83,99,115,131,163,195,227,258};
		static const unsigned long LENEXTRA[29] = {0,0,0,0,0,0,0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4,  4,  5,  5,  5,  5,  0};
		static const unsigned long DISTBASE[30] =  {1,2,3,4,5,7,9,13,17,25,33,49,65,97,129,193,257,385,513,769,1025,1537,2049,3073,4097,6145,8193,12289,16385,24577};
		static const unsigned long DISTEXTRA[30] = {0,0,0,0,1,1,2, 2, 3, 3, 4, 4, 5, 5,  6,  6,  7,  7,  8,  8,   9,   9,  10,  10,  11,  11,  12,   12,   13,   13};
		static const unsigned long CLCL[19] = {16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15}; //code length code lengths
		struct Zlib //nested functions for zlib decompression
		{
			static unsigned long readBitFromStream(size_t& bitp, const unsigned char* bits) { unsigned long result = (bits[bitp >> 3] >> (bitp & 0x7)) & 1; bitp++; return result;}
			static unsigned long readBitsFromStream(size_t& bitp, const unsigned char* bits, size_t nbits)
			{
				unsigned long result = 0;
				for(size_t i = 0; i < nbits; i++) result += (readBitFromStream(bitp, bits)) << i;
				return result;
			}
			struct HuffmanTree
			{
				int makeFromLengths(const std::vector<unsigned long>& bitlen, unsigned long maxbitlen)
				{ //make tree given the lengths
					unsigned long numcodes = (unsigned long)(bitlen.size()), treepos = 0, nodefilled = 0;
					std::vector<unsigned long> tree1d(numcodes), blcount(maxbitlen + 1, 0), nextcode(maxbitlen + 1, 0);
					for(unsigned long bits = 0; bits < numcodes; bits++) blcount[bitlen[bits]]++; //count number of instances of each code length
					for(unsigned long bits = 1; bits <= maxbitlen; bits++) nextcode[bits] = (nextcode[bits - 1] + blcount[bits - 1]) << 1;
					for(unsigned long n = 0; n < numcodes; n++) if(bitlen[n] != 0) tree1d[n] = nextcode[bitlen[n]]++; //generate all the codes
					tree2d.clear(); tree2d.resize(numcodes * 2, 32767); //32767 here means the tree2d isn't filled there yet
					for(unsigned long n = 0; n < numcodes; n++) //the codes
						for(unsigned long i = 0; i < bitlen[n]; i++) //the bits for this code
						{
							unsigned long bit = (tree1d[n] >> (bitlen[n] - i - 1)) & 1;
							if(treepos > numcodes - 2) return 55;
							if(tree2d[2 * treepos + bit] == 32767) //not yet filled in
							{
								if(i + 1 == bitlen[n]) { tree2d[2 * treepos + bit] = n; treepos = 0; } //last bit
								else { tree2d[2 * treepos + bit] = ++nodefilled + numcodes; treepos = nodefilled; } //addresses are encoded as values > numcodes
							}
							else treepos = tree2d[2 * treepos + bit] - numcodes; //subtract numcodes from address to get address value
						}
					return 0;
				}
				int decode(bool& decoded, unsigned long& result, size_t& treepos, unsigned long bit) const
				{ //Decodes a symbol from the tree
					unsigned long numcodes = (unsigned long)tree2d.size() / 2;
					if(treepos >= numcodes) return 11; //error: you appeared outside the codetree
					result = tree2d[2 * treepos + bit];
					decoded = (result < numcodes);
					treepos = decoded ? 0 : result - numcodes;
					return 0;
				}
				std::vector<unsigned long> tree2d; //2D representation of a huffman tree: The one dimension is "0" or "1", the other contains all nodes and leaves of the tree.
			};
			struct Inflator
			{
				int error;
				void inflate(std::vector<unsigned char>& out, const std::vector<unsigned char>& in, size_t inpos = 0)
				{
					size_t bp = 0, pos = 0; //bit pointer and byte pointer
					error = 0;
					unsigned long BFINAL = 0;
					while(!BFINAL && !error)
					{
						if(bp >> 3 >= in.size()) { error = 52; return; } //error, bit pointer will jump past memory
						BFINAL = readBitFromStream(bp, &in[inpos]);
						unsigned long BTYPE = readBitFromStream(bp, &in[inpos]); BTYPE += 2 * readBitFromStream(bp, &in[inpos]);
						if(BTYPE == 3) { error = 20; return; } //error: invalid BTYPE
						else if(BTYPE == 0) inflateNoCompression(out, &in[inpos], bp, pos, in.size());
						else inflateHuffmanBlock(out, &in[inpos], bp, pos, in.size(), BTYPE);
					}
					if(!error) out.resize(pos); //Only now we know the true size of out, resize it to that
				}
				void generateFixedTrees(HuffmanTree& tree, HuffmanTree& treeD) //get the tree of a deflated block with fixed tree
				{
					std::vector<unsigned long> bitlen(288, 8), bitlenD(32, 5);;
					for(size_t i = 144; i <= 255; i++) bitlen[i] = 9;
					for(size_t i = 256; i <= 279; i++) bitlen[i] = 7;
					tree.makeFromLengths(bitlen, 15);
					treeD.makeFromLengths(bitlenD, 15);
				}
				HuffmanTree codetree, codetreeD, codelengthcodetree; //the code tree for Huffman codes, dist codes, and code length codes
				unsigned long huffmanDecodeSymbol(const unsigned char* in, size_t& bp, const HuffmanTree& codetree, size_t inlength)
				{ //decode a single symbol from given list of bits with given code tree. return value is the symbol
					bool decoded; unsigned long ct;
					for(size_t treepos = 0;;)
					{
						if((bp & 0x07) == 0 && (bp >> 3) > inlength) { error = 10; return 0; } //error: end reached without endcode
						error = codetree.decode(decoded, ct, treepos, readBitFromStream(bp, in)); if(error) return 0; //stop, an error happened
						if(decoded) return ct;
					}
				}
				void getTreeInflateDynamic(HuffmanTree& tree, HuffmanTree& treeD, const unsigned char* in, size_t& bp, size_t inlength)
				{ //get the tree of a deflated block with dynamic tree, the tree itself is also Huffman compressed with a known tree
					std::vector<unsigned long> bitlen(288, 0), bitlenD(32, 0);
					if(bp >> 3 >= inlength - 2) { error = 49; return; } //the bit pointer is or will go past the memory
					size_t HLIT =  readBitsFromStream(bp, in, 5) + 257; //number of literal/length codes + 257
					size_t HDIST = readBitsFromStream(bp, in, 5) + 1; //number of dist codes + 1
					size_t HCLEN = readBitsFromStream(bp, in, 4) + 4; //number of code length codes + 4
					std::vector<unsigned long> codelengthcode(19); //lengths of tree to decode the lengths of the dynamic tree
					for(size_t i = 0; i < 19; i++) codelengthcode[CLCL[i]] = (i < HCLEN) ? readBitsFromStream(bp, in, 3) : 0;
					error = codelengthcodetree.makeFromLengths(codelengthcode, 7); if(error) return;
					size_t i = 0, replength;
					while(i < HLIT + HDIST)
					{
						unsigned long code = huffmanDecodeSymbol(in, bp, codelengthcodetree, inlength); if(error) return;
						if(code <= 15)  { if(i < HLIT) bitlen[i++] = code; else bitlenD[i++ - HLIT] = code; } //a length code
						else if(code == 16) //repeat previous
						{
							if(bp >> 3 >= inlength) { error = 50; return; } //error, bit pointer jumps past memory
							replength = 3 + readBitsFromStream(bp, in, 2);
							unsigned long value; //set value to the previous code
							if((i - 1) < HLIT) value = bitlen[i - 1];
							else value = bitlenD[i - HLIT - 1];
							for(size_t n = 0; n < replength; n++) //repeat this value in the next lengths
							{
								if(i >= HLIT + HDIST) { error = 13; return; } //error: i is larger than the amount of codes
								if(i < HLIT) bitlen[i++] = value; else bitlenD[i++ - HLIT] = value;
							}
						}
						else if(code == 17) //repeat "0" 3-10 times
						{
							if(bp >> 3 >= inlength) { error = 50; return; } //error, bit pointer jumps past memory
							replength = 3 + readBitsFromStream(bp, in, 3);
							for(size_t n = 0; n < replength; n++) //repeat this value in the next lengths
							{
								if(i >= HLIT + HDIST) { error = 14; return; } //error: i is larger than the amount of codes
								if(i < HLIT) bitlen[i++] = 0; else bitlenD[i++ - HLIT] = 0;
							}
						}
						else if(code == 18) //repeat "0" 11-138 times
						{
							if(bp >> 3 >= inlength) { error = 50; return; } //error, bit pointer jumps past memory
							replength = 11 + readBitsFromStream(bp, in, 7);
							for(size_t n = 0; n < replength; n++) //repeat this value in the next lengths
							{
								if(i >= HLIT + HDIST) { error = 15; return; } //error: i is larger than the amount of codes
								if(i < HLIT) bitlen[i++] = 0; else bitlenD[i++ - HLIT] = 0;
							}
						}
						else { error = 16; return; } //error: somehow an unexisting code appeared. This can never happen.
					}
					if(bitlen[256] == 0) { error = 64; return; } //the length of the end code 256 must be larger than 0
					error = tree.makeFromLengths(bitlen, 15); if(error) return; //now we've finally got HLIT and HDIST, so generate the code trees, and the function is done
					error = treeD.makeFromLengths(bitlenD, 15); if(error) return;
				}
				void inflateHuffmanBlock(std::vector<unsigned char>& out, const unsigned char* in, size_t& bp, size_t& pos, size_t inlength, unsigned long btype) 
				{
					if(btype == 1) { generateFixedTrees(codetree, codetreeD); }
					else if(btype == 2) { getTreeInflateDynamic(codetree, codetreeD, in, bp, inlength); if(error) return; }
					for(;;)
					{
						unsigned long code = huffmanDecodeSymbol(in, bp, codetree, inlength); if(error) return;
						if(code == 256) return; //end code
						else if(code <= 255) //literal symbol
						{
							if(pos >= out.size()) out.resize((pos + 1) * 2); //reserve more room
							out[pos++] = (unsigned char)(code);
						}
						else if(code >= 257 && code <= 285) //length code
						{
							size_t length = LENBASE[code - 257], numextrabits = LENEXTRA[code - 257];
							if((bp >> 3) >= inlength) { error = 51; return; } //error, bit pointer will jump past memory
							length += readBitsFromStream(bp, in, numextrabits);
							unsigned long codeD = huffmanDecodeSymbol(in, bp, codetreeD, inlength); if(error) return;
							if(codeD > 29) { error = 18; return; } //error: invalid dist code (30-31 are never used)
							unsigned long dist = DISTBASE[codeD], numextrabitsD = DISTEXTRA[codeD];
							if((bp >> 3) >= inlength) { error = 51; return; } //error, bit pointer will jump past memory
							dist += readBitsFromStream(bp, in, numextrabitsD);
							size_t start = pos, back = start - dist; //backwards
							if(pos + length >= out.size()) out.resize((pos + length) * 2); //reserve more room
							for(size_t i = 0; i < length; i++) { out[pos++] = out[back++]; if(back >= start) back = start - dist; }
						}
					}
				}
				void inflateNoCompression(std::vector<unsigned char>& out, const unsigned char* in, size_t& bp, size_t& pos, size_t inlength)
				{
					while((bp & 0x7) != 0) bp++; //go to first boundary of byte
					size_t p = bp / 8;
					if(p >= inlength - 4) { error = 52; return; } //error, bit pointer will jump past memory
					unsigned long LEN = in[p] + 256 * in[p + 1], NLEN = in[p + 2] + 256 * in[p + 3]; p += 4;
					if(LEN + NLEN != 65535) { error = 21; return; } //error: NLEN is not one's complement of LEN
					if(pos + LEN >= out.size()) out.resize(pos + LEN);
					if(p + LEN > inlength) { error = 23; return; } //error: reading outside of in buffer
					for(unsigned long n = 0; n < LEN; n++) out[pos++] = in[p++]; //read LEN bytes of literal data
					bp = p * 8;
				}
			};
			int decompress(std::vector<unsigned char>& out, const std::vector<unsigned char>& in) //returns error value
			{
				Inflator inflator;
				if(in.size() < 2) { return 53; } //error, size of zlib data too small
				if((in[0] * 256 + in[1]) % 31 != 0) { return 24; } //error: 256 * in[0] + in[1] must be a multiple of 31, the FCHECK value is supposed to be made that way
				unsigned long CM = in[0] & 15, CINFO = (in[0] >> 4) & 15, FDICT = (in[1] >> 5) & 1;
				if(CM != 8 || CINFO > 7) { return 25; } //error: only compression method 8: inflate with sliding window of 32k is supported by the PNG spec
				if(FDICT != 0) { return 26; } //error: the specification of PNG says about the zlib stream: "The additional flags shall not specify a preset dictionary."
				inflator.inflate(out, in, 2);
				return inflator.error; //note: adler32 checksum was skipped and ignored
			}
		};
		struct PNG //nested functions for PNG decoding
		{
			struct Info
			{
				unsigned long width, height, colorType, bitDepth, compressionMethod, filterMethod, interlaceMethod, key_r, key_g, key_b;
				bool key_defined; //is a transparent color key given?
				std::vector<unsigned char> palette;
			} info;
			int error;
			void decode(std::vector<unsigned char>& out, const unsigned char* in, size_t size, bool convert_to_rgba32)
			{
				error = 0;
				if(size == 0 || in == 0) { error = 48; return; } //the given data is empty
				readPngHeader(&in[0], size); if(error) return;
				size_t pos = 33; //first byte of the first chunk after the header
				std::vector<unsigned char> idat; //the data from idat chunks
				bool IEND = false, known_type = true;
				info.key_defined = false;
				while(!IEND) //loop through the chunks, ignoring unknown chunks and stopping at IEND chunk. IDAT data is put at the start of the in buffer
				{
					if(pos + 8 >= size) { error = 30; return; } //error: size of the in buffer too small to contain next chunk
					size_t chunkLength = read32bitInt(&in[pos]); pos += 4;
					if(chunkLength > 2147483647) { error = 63; return; }
					if(pos + chunkLength >= size) { error = 35; return; } //error: size of the in buffer too small to contain next chunk
					if(in[pos + 0] == 'I' && in[pos + 1] == 'D' && in[pos + 2] == 'A' && in[pos + 3] == 'T') //IDAT chunk, containing compressed image data
					{
						idat.insert(idat.end(), &in[pos + 4], &in[pos + 4 + chunkLength]);
						pos += (4 + chunkLength);
					}
					else if(in[pos + 0] == 'I' && in[pos + 1] == 'E' && in[pos + 2] == 'N' && in[pos + 3] == 'D')  { pos += 4; IEND = true; }
					else if(in[pos + 0] == 'P' && in[pos + 1] == 'L' && in[pos + 2] == 'T' && in[pos + 3] == 'E') //palette chunk (PLTE)
					{
						pos += 4; //go after the 4 letters
						info.palette.resize(4 * (chunkLength / 3));
						if(info.palette.size() > (4 * 256)) { error = 38; return; } //error: palette too big
						for(size_t i = 0; i < info.palette.size(); i += 4)
						{
							for(size_t j = 0; j < 3; j++) info.palette[i + j] = in[pos++]; //RGB
							info.palette[i + 3] = 255; //alpha
						}
					}
					else if(in[pos + 0] == 't' && in[pos + 1] == 'R' && in[pos + 2] == 'N' && in[pos + 3] == 'S') //palette transparency chunk (tRNS)
					{
						pos += 4; //go after the 4 letters
						if(info.colorType == 3)
						{
							if(4 * chunkLength > info.palette.size()) { error = 39; return; } //error: more alpha values given than there are palette entries
							for(size_t i = 0; i < chunkLength; i++) info.palette[4 * i + 3] = in[pos++];
						}
						else if(info.colorType == 0)
						{
							if(chunkLength != 2) { error = 40; return; } //error: this chunk must be 2 bytes for greyscale image
							info.key_defined = 1; info.key_r = info.key_g = info.key_b = 256 * in[pos] + in[pos + 1]; pos += 2;
						}
						else if(info.colorType == 2)
						{
							if(chunkLength != 6) { error = 41; return; } //error: this chunk must be 6 bytes for RGB image
							info.key_defined = 1;
							info.key_r = 256 * in[pos] + in[pos + 1]; pos += 2;
							info.key_g = 256 * in[pos] + in[pos + 1]; pos += 2;
							info.key_b = 256 * in[pos] + in[pos + 1]; pos += 2;
						}
						else { error = 42; return; } //error: tRNS chunk not allowed for other color models
					}
					else //it's not an implemented chunk type, so ignore it: skip over the data
					{
						if(!(in[pos + 0] & 32)) { error = 69; return; } //error: unknown critical chunk (5th bit of first byte of chunk type is 0)
						pos += (chunkLength + 4); //skip 4 letters and uninterpreted data of unimplemented chunk
						known_type = false;
					}
					pos += 4; //step over CRC (which is ignored)
				}
				unsigned long bpp = getBpp(info);
				std::vector<unsigned char> scanlines(((info.width * (info.height * bpp + 7)) / 8) + info.height); //now the out buffer will be filled
				Zlib zlib; //decompress with the Zlib decompressor
				error = zlib.decompress(scanlines, idat); if(error) return; //stop if the zlib decompressor returned an error
				size_t bytewidth = (bpp + 7) / 8, outlength = (info.height * info.width * bpp + 7) / 8;
				out.resize(outlength); //time to fill the out buffer
				unsigned char* out_ = outlength ? &out[0] : 0; //use a regular pointer to the std::vector for faster code if compiled without optimization
				if(info.interlaceMethod == 0) //no interlace, just filter
				{
					size_t linestart = 0, linelength = (info.width * bpp + 7) / 8; //length in bytes of a scanline, excluding the filtertype byte
					if(bpp >= 8) //byte per byte
						for(unsigned long y = 0; y < info.height; y++)
						{
							unsigned long filterType = scanlines[linestart];
							const unsigned char* prevline = (y == 0) ? 0 : &out_[(y - 1) * info.width * bytewidth];
							unFilterScanline(&out_[linestart - y], &scanlines[linestart + 1], prevline, bytewidth, filterType,  linelength); if(error) return;
							linestart += (1 + linelength); //go to start of next scanline
						}
					else //less than 8 bits per pixel, so fill it up bit per bit
					{
						std::vector<unsigned char> templine((info.width * bpp + 7) >> 3); //only used if bpp < 8
						for(size_t y = 0, obp = 0; y < info.height; y++)
						{
							unsigned long filterType = scanlines[linestart];
							const unsigned char* prevline = (y == 0) ? 0 : &out_[(y - 1) * info.width * bytewidth];
							unFilterScanline(&templine[0], &scanlines[linestart + 1], prevline, bytewidth, filterType, linelength); if(error) return;
							for(size_t bp = 0; bp < info.width * bpp;) setBitOfReversedStream(obp, out_, readBitFromReversedStream(bp, &templine[0]));
							linestart += (1 + linelength); //go to start of next scanline
						}
					}
				}
				else //interlaceMethod is 1 (Adam7)
				{
					size_t passw[7] = { (info.width + 7) / 8, (info.width + 3) / 8, (info.width + 3) / 4, (info.width + 1) / 4, (info.width + 1) / 2, (info.width + 0) / 2, (info.width + 0) / 1 };
					size_t passh[7] = { (info.height + 7) / 8, (info.height + 7) / 8, (info.height + 3) / 8, (info.height + 3) / 4, (info.height + 1) / 4, (info.height + 1) / 2, (info.height + 0) / 2 };
					size_t passstart[7] = {0};
					size_t pattern[28] = {0,4,0,2,0,1,0,0,0,4,0,2,0,1,8,8,4,4,2,2,1,8,8,8,4,4,2,2}; //values for the adam7 passes
					for(int i = 0; i < 6; i++) passstart[i + 1] = passstart[i] + passh[i] * ((passw[i] ? 1 : 0) + (passw[i] * bpp + 7) / 8);
					std::vector<unsigned char> scanlineo((info.width * bpp + 7) / 8), scanlinen((info.width * bpp + 7) / 8); //"old" and "new" scanline
					for(int i = 0; i < 7; i++)
						adam7Pass(&out_[0], &scanlinen[0], &scanlineo[0], &scanlines[passstart[i]], info.width, pattern[i], pattern[i + 7], pattern[i + 14], pattern[i + 21], passw[i], passh[i], bpp);
				}
				if(convert_to_rgba32 && (info.colorType != 6 || info.bitDepth != 8)) //conversion needed
				{
					std::vector<unsigned char> data = out;
					error = convert(out, &data[0], info, info.width, info.height);
				}
			}
			void readPngHeader(const unsigned char* in, size_t inlength) //read the information from the header and store it in the Info
			{
				if(inlength < 29) { error = 27; return; } //error: the data length is smaller than the length of the header
				if(in[0] != 137 || in[1] != 80 || in[2] != 78 || in[3] != 71 || in[4] != 13 || in[5] != 10 || in[6] != 26 || in[7] != 10) { error = 28; return; } //no PNG signature
				if(in[12] != 'I' || in[13] != 'H' || in[14] != 'D' || in[15] != 'R') { error = 29; return; } //error: it doesn't start with a IHDR chunk!
				info.width = read32bitInt(&in[16]); info.height = read32bitInt(&in[20]);
				info.bitDepth = in[24]; info.colorType = in[25];
				info.compressionMethod = in[26]; if(in[26] != 0) { error = 32; return; } //error: only compression method 0 is allowed in the specification
				info.filterMethod = in[27]; if(in[27] != 0) { error = 33; return; } //error: only filter method 0 is allowed in the specification
				info.interlaceMethod = in[28]; if(in[28] > 1) { error = 34; return; } //error: only interlace methods 0 and 1 exist in the specification
				error = checkColorValidity(info.colorType, info.bitDepth);
			}
			void unFilterScanline(unsigned char* recon, const unsigned char* scanline, const unsigned char* precon, size_t bytewidth, unsigned long filterType, size_t length)
			{
				switch(filterType)
				{
					case 0: for(size_t i = 0; i < length; i++) recon[i] = scanline[i]; break;
					case 1:
						for(size_t i =         0; i < bytewidth; i++) recon[i] = scanline[i];
						for(size_t i = bytewidth; i <    length; i++) recon[i] = scanline[i] + recon[i - bytewidth];
						break;
					case 2:
						if(precon) for(size_t i = 0; i < length; i++) recon[i] = scanline[i] + precon[i];
						else       for(size_t i = 0; i < length; i++) recon[i] = scanline[i];
						break;
					case 3:
						if(precon)
						{
							for(size_t i =         0; i < bytewidth; i++) recon[i] = scanline[i] + precon[i] / 2;
							for(size_t i = bytewidth; i <    length; i++) recon[i] = scanline[i] + ((recon[i - bytewidth] + precon[i]) / 2);
						}
						else
						{
							for(size_t i =         0; i < bytewidth; i++) recon[i] = scanline[i];
							for(size_t i = bytewidth; i <    length; i++) recon[i] = scanline[i] + recon[i - bytewidth] / 2;
						}
						break;
					case 4:
						if(precon)
						{
							for(size_t i =         0; i < bytewidth; i++) recon[i] = scanline[i] + paethPredictor(0, precon[i], 0);
							for(size_t i = bytewidth; i <    length; i++) recon[i] = scanline[i] + paethPredictor(recon[i - bytewidth], precon[i], precon[i - bytewidth]);
						}
						else
						{
							for(size_t i =         0; i < bytewidth; i++) recon[i] = scanline[i];
							for(size_t i = bytewidth; i <    length; i++) recon[i] = scanline[i] + paethPredictor(recon[i - bytewidth], 0, 0);
						}
						break;
					default: error = 36; return; //error: unexisting filter type given
				}
			}
			void adam7Pass(unsigned char* out, unsigned char* linen, unsigned char* lineo, const unsigned char* in, unsigned long w, size_t passleft, size_t passtop, size_t spacex, size_t spacey, size_t passw, size_t passh, unsigned long bpp)
			{ //filter and reposition the pixels into the output when the image is Adam7 interlaced. This function can only do it after the full image is already decoded. The out buffer must have the correct allocated memory size already.
				if(passw == 0) return;
				size_t bytewidth = (bpp + 7) / 8, linelength = 1 + ((bpp * passw + 7) / 8);
				for(unsigned long y = 0; y < passh; y++)
				{
					unsigned char filterType = in[y * linelength], *prevline = (y == 0) ? 0 : lineo;
					unFilterScanline(linen, &in[y * linelength + 1], prevline, bytewidth, filterType, (w * bpp + 7) / 8); if(error) return;
					if(bpp >= 8) for(size_t i = 0; i < passw; i++) for(size_t b = 0; b < bytewidth; b++) //b = current byte of this pixel
						out[bytewidth * w * (passtop + spacey * y) + bytewidth * (passleft + spacex * i) + b] = linen[bytewidth * i + b];
					else for(size_t i = 0; i < passw; i++)
					{
						size_t obp = bpp * w * (passtop + spacey * y) + bpp * (passleft + spacex * i), bp = i * bpp;
						for(size_t b = 0; b < bpp; b++) setBitOfReversedStream(obp, out, readBitFromReversedStream(bp, &linen[0]));
					}
					unsigned char* temp = linen; linen = lineo; lineo = temp; //swap the two buffer pointers "line old" and "line new"
				}
			}
			static unsigned long readBitFromReversedStream(size_t& bitp, const unsigned char* bits) { unsigned long result = (bits[bitp >> 3] >> (7 - (bitp & 0x7))) & 1; bitp++; return result;}
			static unsigned long readBitsFromReversedStream(size_t& bitp, const unsigned char* bits, unsigned long nbits)
			{
				unsigned long result = 0;
				for(size_t i = nbits - 1; i < nbits; i--) result += ((readBitFromReversedStream(bitp, bits)) << i);
				return result;
			}
			void setBitOfReversedStream(size_t& bitp, unsigned char* bits, unsigned long bit) { bits[bitp >> 3] |=  (bit << (7 - (bitp & 0x7))); bitp++; }
			unsigned long read32bitInt(const unsigned char* buffer) { return (buffer[0] << 24) | (buffer[1] << 16) | (buffer[2] << 8) | buffer[3]; }
			int checkColorValidity(unsigned long colorType, unsigned long bd) //return type is a LodePNG error code
			{
				if((colorType == 2 || colorType == 4 || colorType == 6)) { if(!(bd == 8 || bd == 16)) return 37; else return 0; }
				else if(colorType == 0) { if(!(bd == 1 || bd == 2 || bd == 4 || bd == 8 || bd == 16)) return 37; else return 0; }
				else if(colorType == 3) { if(!(bd == 1 || bd == 2 || bd == 4 || bd == 8            )) return 37; else return 0; }
				else return 31; //unexisting color type
			}
			unsigned long getBpp(const Info& info)
			{
				if(info.colorType == 2) return (3 * info.bitDepth);
				else if(info.colorType >= 4) return (info.colorType - 2) * info.bitDepth;
				else return info.bitDepth;
			}
			int convert(std::vector<unsigned char>& out, const unsigned char* in, Info& infoIn, unsigned long w, unsigned long h)
			{ //converts from any color type to 32-bit. return value = LodePNG error code
				size_t numpixels = w * h, bp = 0;
				out.resize(numpixels * 4);
				unsigned char* out_ = out.empty() ? 0 : &out[0]; //faster if compiled without optimization
				if(infoIn.bitDepth == 8 && infoIn.colorType == 0) //greyscale
					for(size_t i = 0; i < numpixels; i++)
					{
						out_[4 * i + 0] = out_[4 * i + 1] = out_[4 * i + 2] = in[i];
						out_[4 * i + 3] = (infoIn.key_defined && in[i] == infoIn.key_r) ? 0 : 255;
					}
				else if(infoIn.bitDepth == 8 && infoIn.colorType == 2) //RGB color
					for(size_t i = 0; i < numpixels; i++)
					{
						for(size_t c = 0; c < 3; c++) out_[4 * i + c] = in[3 * i + c];
						out_[4 * i + 3] = (infoIn.key_defined == 1 && in[3 * i + 0] == infoIn.key_r && in[3 * i + 1] == infoIn.key_g && in[3 * i + 2] == infoIn.key_b) ? 0 : 255;
					}
				else if(infoIn.bitDepth == 8 && infoIn.colorType == 3) //indexed color (palette)
					for(size_t i = 0; i < numpixels; i++)
					{
						if(4U * in[i] >= infoIn.palette.size()) return 46;
						for(size_t c = 0; c < 4; c++) out_[4 * i + c] = infoIn.palette[4 * in[i] + c]; //get rgb colors from the palette
					}
				else if(infoIn.bitDepth == 8 && infoIn.colorType == 4) //greyscale with alpha
					for(size_t i = 0; i < numpixels; i++)
					{
						out_[4 * i + 0] = out_[4 * i + 1] = out_[4 * i + 2] = in[2 * i + 0];
						out_[4 * i + 3] = in[2 * i + 1];
					}
				else if(infoIn.bitDepth == 8 && infoIn.colorType == 6) for(size_t i = 0; i < numpixels; i++) for(size_t c = 0; c < 4; c++) out_[4 * i + c] = in[4 * i + c]; //RGB with alpha
				else if(infoIn.bitDepth == 16 && infoIn.colorType == 0) //greyscale
					for(size_t i = 0; i < numpixels; i++)
					{
						out_[4 * i + 0] = out_[4 * i + 1] = out_[4 * i + 2] = in[2 * i];
						out_[4 * i + 3] = (infoIn.key_defined && 256U * in[i] + in[i + 1] == infoIn.key_r) ? 0 : 255;
					}
				else if(infoIn.bitDepth == 16 && infoIn.colorType == 2) //RGB color
					for(size_t i = 0; i < numpixels; i++)
					{
						for(size_t c = 0; c < 3; c++) out_[4 * i + c] = in[6 * i + 2 * c];
						out_[4 * i + 3] = (infoIn.key_defined && 256U*in[6*i+0]+in[6*i+1] == infoIn.key_r && 256U*in[6*i+2]+in[6*i+3] == infoIn.key_g && 256U*in[6*i+4]+in[6*i+5] == infoIn.key_b) ? 0 : 255;
					}
				else if(infoIn.bitDepth == 16 && infoIn.colorType == 4) //greyscale with alpha
					for(size_t i = 0; i < numpixels; i++)
					{
						out_[4 * i + 0] = out_[4 * i + 1] = out_[4 * i + 2] = in[4 * i]; //most significant byte
						out_[4 * i + 3] = in[4 * i + 2];
					}
				else if(infoIn.bitDepth == 16 && infoIn.colorType == 6) for(size_t i = 0; i < numpixels; i++) for(size_t c = 0; c < 4; c++) out_[4 * i + c] = in[8 * i + 2 * c]; //RGB with alpha
				else if(infoIn.bitDepth < 8 && infoIn.colorType == 0) //greyscale
					for(size_t i = 0; i < numpixels; i++)
					{
						unsigned long value = (readBitsFromReversedStream(bp, in, infoIn.bitDepth) * 255) / ((1 << infoIn.bitDepth) - 1); //scale value from 0 to 255
						out_[4 * i + 0] = out_[4 * i + 1] = out_[4 * i + 2] = (unsigned char)(value);
						out_[4 * i + 3] = (infoIn.key_defined && value && ((1U << infoIn.bitDepth) - 1U) == infoIn.key_r && ((1U << infoIn.bitDepth) - 1U)) ? 0 : 255;
					}
				else if(infoIn.bitDepth < 8 && infoIn.colorType == 3) //palette
					for(size_t i = 0; i < numpixels; i++)
					{
						unsigned long value = readBitsFromReversedStream(bp, in, infoIn.bitDepth);
						if(4 * value >= infoIn.palette.size()) return 47;
						for(size_t c = 0; c < 4; c++) out_[4 * i + c] = infoIn.palette[4 * value + c]; //get rgb colors from the palette
					}
				return 0;
			}
			unsigned char paethPredictor(short a, short b, short c) //Paeth predicter, used by PNG filter type 4
			{
				short p = a + b - c, pa = p > a ? (p - a) : (a - p), pb = p > b ? (p - b) : (b - p), pc = p > c ? (p - c) : (c - p);
				return (unsigned char)((pa <= pb && pa <= pc) ? a : pb <= pc ? b : c);
			}
		};
		PNG decoder; decoder.decode(out_image, in_png, in_size, convert_to_rgba32);
		image_width = decoder.info.width; image_height = decoder.info.height;
		return decoder.error;
	}
	
	//======================================================================================================
	void loadPNGTexture( const std::string& PNGFilename, int width = 256, int height = 256, int bitdepth =32 ) {
		
		std::vector<unsigned char> buffer, image;

		loadFile(buffer, PNGFilename );

		unsigned long w, h;
		int error = decodePNG(image, w, h, buffer.empty() ? 0 : &buffer[0], (unsigned long)buffer.size());
		
		//if there's an error, display it
		if(error != 0) std::cout << "error: " << error << std::endl;
		
		//the pixels are now in the vector "image", use it as texture, draw it, ...
		if(image.size() > 4) std::cout << "width: " << w << " height: " << h << " first pixel: " << std::hex << int(image[0]) << int(image[1]) << int(image[2]) << int(image[3]) << std::endl;

		// allocate a texture name
		glGenTextures( 1, &Tname );
			
		// select our current texture
		glBindTexture( GL_TEXTURE_2D, Tname );
			
		// select modulate to mix texture with color for shading
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
			
		// when texture area is small, bilinear filter the closest mipmap
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST );
		// when texture area is large, bilinear filter the first mipmap
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			
		// if wrap is true, the texture wraps over at the edges (repeat)
		//       ... false, the texture ends at the edges (clamp)
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
#else
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
		
		// build our texture mipmaps
		gluBuild2DMipmaps( GL_TEXTURE_2D, 4, w, h, GL_RGBA, GL_UNSIGNED_BYTE, &image );
#endif
	}

	//======================================================================================================
	void loadTexture( const std::string& filename ) {
		//Tombstone TTexture system
		string	extension = "";
		long	dotLocation;
	 
		//get location of . in filename to load
		dotLocation = InStr( filename, ".", filename.size()-5 );
		if ( dotLocation != -1 ) {
			//get the extension
			extension = Right( filename, filename.size() -dotLocation );
		}
		
		cout << "LOADING:[" << filename << "] extension = " << extension << " ... ";
		if ( hasEnding( filename, ".png" ) == TRUE ) {
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
			//Apple loader for iPhone and OSX platform
			NSUInteger				width,								//Width of an image
			height,														//height of an image
			i;
			CGContextRef			context = nil;						//its context
			void*					data = nil;;						//the image data
			CGColorSpaceRef			colorSpace;							//its colour space reference
			void*					tempData;
			unsigned int*			inPixel32;
			unsigned short*			outPixel16;
			short					hasAlpha;
			CGImageAlphaInfo		info;								//image alpha information
			CGAffineTransform		transform;
			CGSize					imageSize;
			Texture2DPixelFormat    pixelFormat;
			CGImageRef				image;
			UIImageOrientation		orientation;
			short					sizeToFit = NO;
	 
			string					ext = getExtension( filename );
			string					nameOfFile ( filename.begin(), filename.end() -ext.size() );
			CFStringRef				Name;
			CFStringRef				type;			
	 
			if ( hasEnding( ext, ".png" ) == TRUE ) {
				//image is a png now convert the strings to CFStringRef for 'Apple'
				cout << " DECODING ... ";
				Name = CFStringCreateWithCString( kCFAllocatorDefault, 
												 nameOfFile.c_str(), 
												 kCFStringEncodingASCII);
				type = CFSTR( "png" );
			}			
	 
			// copy from a CFString into a local c string
			image = CreateNamedImage( Name, type );					//create and LOAD the image this is then returned
			orientation = UIImageOrientationUp;							//adjust orientation
			if(image == NULL) {
				cout << "Image is Null [" << gNumTextures << "] <" << filename << ">" << endl;
				cout << "Please check filename as null error can be incorrect filename" << endl;
			}
	 
			info = CGImageGetAlphaInfo(image);
			hasAlpha = ((info == kCGImageAlphaPremultipliedLast) || (info == kCGImageAlphaPremultipliedFirst) || (info == kCGImageAlphaLast) || (info == kCGImageAlphaFirst) ? YES : NO);
			if(CGImageGetColorSpace(image)) {
				if(hasAlpha)
					pixelFormat = kTexture2DPixelFormat_RGBA8888;
				else
					pixelFormat = kTexture2DPixelFormat_RGB565;
			} else { //NOTE: No colorspace means a mask image
				pixelFormat = kTexture2DPixelFormat_A8;
				imageSize.width *= 0.5;
			}
	 
			imageSize = CGSizeMake(CGImageGetWidth(image), CGImageGetHeight(image));
			transform = CGAffineTransformIdentity;
	 
			width = imageSize.width;
			// round up to nearest power of 2
			if((width != 1) && (width & (width - 1))) {
				i = 1;
				while((sizeToFit ? 2 * i : i) < width)
					i *= 2;
				width = i;
			}
			height = imageSize.height;
			// round up to nearest power of 2
			if((height != 1) && (height & (height - 1))) {
				i = 1;
				while((sizeToFit ? 2 * i : i) < height)
					i *= 2;
				height = i;
			}
	 
			// check texture size is within maximum texture size (1024x1024), although if you're loading in textures this big you're going to have memory issues anyway
			while((width > kMaxTextureSize) || (height > kMaxTextureSize)) {
				width /= 2;
				height /= 2;
				transform = CGAffineTransformScale(transform, 0.5, 0.5);	
				imageSize.width *= 0.5;
				imageSize.height *= 0.5;
			}
	 
			//setup the pixel format
			switch(pixelFormat) {		
				case kTexture2DPixelFormat_RGBA8888:
					colorSpace = CGColorSpaceCreateDeviceRGB();
					data = calloc(height * width * 4,1);
					context = CGBitmapContextCreate(data, width, height, 8, 4 * width, colorSpace, kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big);
					CGColorSpaceRelease(colorSpace); break;
				case kTexture2DPixelFormat_RGB565:
					colorSpace = CGColorSpaceCreateDeviceRGB();
					data = calloc(height * width * 4,1);
					context = CGBitmapContextCreate(data, width, height, 8, 4 * width, colorSpace, kCGImageAlphaNoneSkipLast | kCGBitmapByteOrder32Big);
					CGColorSpaceRelease(colorSpace); break;
				case kTexture2DPixelFormat_A8:
					data = calloc(height * width,1);
					context = CGBitmapContextCreate(data, width, height, 8, width, NULL, kCGImageAlphaOnly); break;				
				default:
					printf( "Invalid texture format\n" );
					exit( 1 );
			}

			CGContextClearRect(context, CGRectMake(0, 0, width, height));
			CGContextTranslateCTM(context, 0, height - imageSize.height);
	 
			if(!CGAffineTransformIsIdentity(transform))
				CGContextConcatCTM(context, transform);
			
			CGContextDrawImage(context, CGRectMake(0, 0, CGImageGetWidth(image), CGImageGetHeight(image)), image);
	 
			//Convert "RRRRRRRRRGGGGGGGGBBBBBBBBAAAAAAAA" to "RRRRRGGGGGGBBBBB"
			if(pixelFormat == kTexture2DPixelFormat_RGB565) {
				tempData = calloc(height * width * 2,1);
				inPixel32 = (unsigned int*)data;
				outPixel16 = (unsigned short*)tempData;

				for(i = 0; i < width * height; ++i, ++inPixel32)
					*outPixel16++ = ((((*inPixel32 >> 0) & 0xFF) >> 3) << 11) | ((((*inPixel32 >> 8) & 0xFF) >> 2) << 5) | ((((*inPixel32 >> 16) & 0xFF) >> 3) << 0);
				
				free(data);
				data = tempData;
			}
	 
			//need to locate a free entry for the texture system to stop any problems
	 
			//		printf("texture loaded, w/h = %d,%d to  %li\n",width,height,gNumTextures);
	 
			//create the entry
			InitTexture(data,pixelFormat,width,height,imageSize );
	 
			CGContextRelease(context);
			free(data);
			CGImageRelease(image);
			cout << "COMPLETED" << endl;
	 #else
			loadPNGTexture( filename );
			if ( Tname == 0x0 ) {
				cout << "libPNG Load texture error " << filename << endl;
				exit( 1 );
			}
	 #endif
		}
	}

	//======================================================================================================
	void randomizeTexture( int OP = FULLSCREEN_NOCHANGE, unsigned char opValue = 255 ) {
		// Copy buffer to texture
		unsigned char* pixelData;
		
		pixelData = new unsigned char[ TombstoneWidth *TombstoneHeight *4 ];
		if ( pixelData == NULL ) {
			cout << "grabTexture:ERROR could not allocate buffer" << endl;
			return;
		}
		
		glBindTexture( GL_TEXTURE_2D, Tname );				//bind the texture for use
		glReadPixels( 0, 0, TombstoneWidth, TombstoneHeight, GL_RGBA, GL_UNSIGNED_BYTE, pixelData );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	//setup the texture parameters
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		
		switch ( OP ) {
			case FULLSCREEN_RANDOMRED:
				for ( long pixPos = 0; pixPos < ( TombstoneWidth *TombstoneHeight) *4; pixPos += 4 ) {
					pixelData[ pixPos ] = TombstoneRandom.randInt( 0, opValue );
				}				
				break;
			case FULLSCREEN_RANDOMGREEN:
				for ( long pixPos = 1; pixPos < ( TombstoneWidth *TombstoneHeight) *4; pixPos += 4 ) {
					pixelData[ pixPos ] = TombstoneRandom.randInt( 0, opValue );
				}				
				break;
			case FULLSCREEN_RANDOMBLUE:
				for ( long pixPos = 2; pixPos < ( TombstoneWidth *TombstoneHeight) *4; pixPos += 4 ) {
					pixelData[ pixPos ] = TombstoneRandom.randInt( 0, opValue );
				}				
				break;
			case FULLSCREEN_RANDOMALPHA:
				for ( long pixPos = 3; pixPos < ( TombstoneWidth *TombstoneHeight) *4; pixPos += 4 ) {
					pixelData[ pixPos ] = TombstoneRandom.randInt( 0, opValue );
				}				
				break;				
			default:
				break;
		}
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, TombstoneWidth, TombstoneHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixelData );
		Twidth = TombstoneWidth;
		Theight = TombstoneHeight;
		
		delete []pixelData;		
	}

	//======================================================================================================
	void grabTexture( long x, long y, long width, long height, long detailLevel = 0, long xoffset = 0, long yoffset = 0 ) {
		// Copy buffer to texture
		unsigned char* pixelData;
		if ( width <64 || height<64 ) {
			cout << "GRAB TEXTURE: illegal image size must be greater than 64 pixels in width and height" << endl;
			return;
		}

		if ( Tname == 0x0 ) {
			//create new texture if required.
			cout << "GRAB: deleting old texture" << endl;
		}
		
		pixelData = new unsigned char[ width *height *4 ];
		if ( pixelData == NULL ) {
			cout << "grabTexture:ERROR could not allocate buffer" << endl;
			return;
		}
		
		glBindTexture( GL_TEXTURE_2D, Tname );				//bind the texture for use
		glReadPixels( x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixelData );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	//setup the texture parameters
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixelData );
		Twidth = width;
		Theight = height;
		
		delete []pixelData;
	}

	//======================================================================================================
	void FreeTexture( void ) {
		glDeleteTextures( 1, &Tname );
		Tname = 0x0;
		texturesBound--;
	}
	
	//======================================================================================================
	bool hasEnding ( const std::string& fullString, string const &ending )
	{
		if (fullString.length() >= ending.length()) {
			return (0 == fullString.compare (fullString.length() - ending.length(), ending.length(), ending));
		} else {
			return false;
		}
	}
	
#if TARGET_OS_IPHONE || TARGET_OS_SIMULATOR
	//======================================================================================================
	CGImageRef CreateNamedImage(CFStringRef imageName, CFStringRef type) {
		//called first to create the basics
		CGImageRef img = NULL;
		
		CFURLRef url = CFBundleCopyResourceURL(CFBundleGetMainBundle(),imageName,type,NULL);
		if(url != NULL)
		{
			CGDataProviderRef imgSrc = CGDataProviderCreateWithURL(url);
			if(imgSrc != NULL)
			{
				img = CGImageCreateWithPNGDataProvider(imgSrc,NULL,false,kCGRenderingIntentDefault);
				CFRelease(imgSrc);
			}
			else
			{
				printf("imgSrc = null for %s.%s\n",CFStringGetCStringPtr(imageName,kCFStringEncodingASCII),CFStringGetCStringPtr(type,kCFStringEncodingASCII));
			}
			CFRelease(url);
		}
		else
		{
			printf("url = null for %s.%s\n",CFStringGetCStringPtr(imageName,kCFStringEncodingASCII),CFStringGetCStringPtr(type,kCFStringEncodingASCII));
		}
		return img;
	}
#else
	void CreateNamedImage( string imageNamed, string type ) {
	}
#endif
	
	//======================================================================================================
	void InitTexture(const void* data, Texture2DPixelFormat pixelFormat, NSUInteger width, NSUInteger height, CGSize size ){
		//textureClass* ret = new textureClass;
		glGenTextures(1, &Tname);											//generate a texture object in OpenGL
		glBindTexture(GL_TEXTURE_2D, Tname);								//bind the object (in 2d)
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );	//setup the texture parameters
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		
		//THIS CREATES THE TEXTURE FOR USAGE
		switch(pixelFormat) 
		{
			case kTexture2DPixelFormat_RGBA8888:
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
				break;
			case kTexture2DPixelFormat_RGB565:
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, data);
				break;
			case kTexture2DPixelFormat_A8:
				glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, width, height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, data);
				break;
			default:
				printf( "Invalid texture format\n" );
				exit( 1 );
				
		}
		
		//setup its details
		TID = gNumTextures;
		Tsize = size;
		Twidth = width;
		Theight = height;
		Tformat = pixelFormat;
		TmaxS = size.width / (float)width;
		TmaxT = size.height / (float)height;
		
		gNumTextures++;
	}

};

typedef TTextureClass TTexture;
